# -*- coding: UTF-8 -*-
"""
Name: filter_denoiser.py
Porpose: Show dialog to get denoiser data based on FFmpeg syntax
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import webbrowser
import wx


class Denoisers(wx.Dialog):
    """
    A dialog tool to get video denoiser values
    based on FFmpeg syntax.

    """
    get = wx.GetApp()
    appdata = get.appset

    def __init__(self, parent, denoiser, iconreset):
        """
        Make sure you use the clear button when you finish the task.
        Enable filters denoiser useful in some case, example when apply
        a deinterlace filter
        <https://askubuntu.com/questions/866186/how-to-get-good-quality-when-
        converting-digital-video>

        """
        if denoiser:
            self.denoiser = denoiser
        else:
            self.denoiser = ''

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizer_base = wx.BoxSizer(wx.VERTICAL)
        sbox = wx.StaticBox(self, wx.ID_ANY, (_("Denoiser Filters")))
        zone = wx.StaticBoxSizer(sbox, wx.VERTICAL)
        sizer_base.Add(zone, 1, wx.ALL | wx.EXPAND, 5)
        self.ckbx_nlmeans = wx.CheckBox(self, wx.ID_ANY,
                                        (_("Enable nlmeans denoiser"))
                                        )
        grid_den = wx.FlexGridSizer(2, 2, 0, 0)
        zone.Add(grid_den)
        grid_den.Add(self.ckbx_nlmeans, 0, wx.ALL
                     | wx.ALIGN_CENTER_VERTICAL
                     | wx.ALIGN_CENTER_HORIZONTAL,
                     5)
        nlmeans = [("Default"),
                   ("Old VHS tapes - good starting point restoration"),
                   ("Heavy - really noisy inputs"),
                   ("Light - good quality inputs")
                   ]
        self.rdb_nlmeans = wx.RadioBox(self, wx.ID_ANY, (_("nlmeans options")),
                                       choices=nlmeans, majorDimension=0,
                                       style=wx.RA_SPECIFY_ROWS
                                       )
        grid_den.Add(self.rdb_nlmeans, 0, wx.ALL
                     | wx.ALIGN_CENTER_VERTICAL
                     | wx.ALIGN_CENTER_HORIZONTAL,
                     5)
        self.ckbx_hqdn3d = wx.CheckBox(self, wx.ID_ANY,
                                       (_("Enable hqdn3d denoiser"))
                                       )
        grid_den.Add(self.ckbx_hqdn3d, 0, wx.ALL
                     | wx.ALIGN_CENTER_VERTICAL
                     | wx.ALIGN_CENTER_HORIZONTAL,
                     5)
        hqdn3d = [("Default"), ("Conservative [4.0:4.0:3.0:3.0]"),
                  ("Old VHS tapes restoration [9.0:5.0:3.0:3.0]")
                  ]
        self.rdb_hqdn3d = wx.RadioBox(self, wx.ID_ANY, (_("hqdn3d options")),
                                      choices=hqdn3d, majorDimension=0,
                                      style=wx.RA_SPECIFY_ROWS
                                      )
        grid_den.Add(self.rdb_hqdn3d, 0, wx.ALL
                     | wx.ALIGN_CENTER_VERTICAL
                     | wx.ALIGN_CENTER_HORIZONTAL,
                     5)
        # ----- confirm buttons section
        gridbtns = wx.GridSizer(1, 2, 0, 0)
        gridhelp = wx.GridSizer(1, 1, 0, 0)
        btn_help = wx.Button(self, wx.ID_HELP, "")
        gridhelp.Add(btn_help, 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
        gridbtns.Add(gridhelp)
        boxaff = wx.BoxSizer(wx.HORIZONTAL)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        boxaff.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        boxaff.Add(btn_ok, 0, wx.LEFT, 5)
        btn_reset = wx.Button(self, wx.ID_ANY, _("Reset"))
        btn_reset.SetBitmap(iconreset, wx.LEFT)
        boxaff.Add(btn_reset, 0, wx.LEFT, 5)
        gridbtns.Add(boxaff, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        sizer_base.Add(gridbtns, 0, wx.EXPAND)
        # ----- Set properties
        self.SetTitle(_("Denoiser Tool"))
        tool = _('nlmeans:\nDenoise frames using Non-Local Means algorithm '
                 'is capable of restoring video sequences, even with strong '
                 'noise. It is ideal for enhancing the quality of old VHS '
                 'tapes.')
        self.ckbx_nlmeans.SetToolTip(tool)
        tool = _('hqdn3d:\nThis is a high precision/quality 3d denoise '
                 'filter. It aims to reduce image noise, producing smooth '
                 'images and making still images really still. It should '
                 'enhance compressibility.')
        self.ckbx_hqdn3d.SetToolTip(tool)
        # ----- Set layout
        self.SetSizer(sizer_base)
        sizer_base.Fit(self)
        self.Layout()

        # ----------------------Binding (EVT)--------------------------#
        self.Bind(wx.EVT_CHECKBOX, self.on_nlmeans, self.ckbx_nlmeans)
        self.Bind(wx.EVT_CHECKBOX, self.on_hqdn3d, self.ckbx_hqdn3d)
        self.Bind(wx.EVT_RADIOBOX, self.on_nlmeans_opt, self.rdb_nlmeans)
        self.Bind(wx.EVT_RADIOBOX, self.on_hqdn3d_opt, self.rdb_hqdn3d)
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_help)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)

        self.settings()

    def settings(self):
        """
        Set default or set in according with previusly activated option
        """
        if self.denoiser:
            if self.denoiser.startswith('nlmeans'):
                spl = self.denoiser.split('=')
                if len(spl) == 1:
                    self.rdb_nlmeans.SetSelection(0)
                else:
                    if spl[1] == '8:3:2':
                        self.rdb_nlmeans.SetSelection(1)
                    if spl[1] == '10:5:3':
                        self.rdb_nlmeans.SetSelection(2)
                    if spl[1] == '6:3:1':
                        self.rdb_nlmeans.SetSelection(3)
                self.ckbx_nlmeans.SetValue(True)
                self.ckbx_hqdn3d.SetValue(False)
                self.ckbx_nlmeans.Enable()
                self.ckbx_hqdn3d.Disable()
                self.rdb_nlmeans.Enable()
                self.rdb_hqdn3d.Disable()

            elif self.denoiser.startswith('hqdn3d'):
                spl = self.denoiser.split('=')
                if len(spl) == 1:
                    self.rdb_hqdn3d.SetSelection(0)
                else:
                    if spl[1] == '4.0:4.0:3.0:3.0':
                        self.rdb_hqdn3d.SetSelection(1)
                    if spl[1] == '9.0:5.0:3.0:3.0':
                        self.rdb_hqdn3d.SetSelection(2)

                self.ckbx_nlmeans.SetValue(False)
                self.ckbx_hqdn3d.SetValue(True)
                self.ckbx_nlmeans.Disable()
                self.ckbx_hqdn3d.Enable()
                self.rdb_nlmeans.Disable()
                self.rdb_hqdn3d.Enable()
        else:
            self.ckbx_nlmeans.SetValue(False)
            self.ckbx_hqdn3d.SetValue(False)
            self.ckbx_nlmeans.Enable()
            self.ckbx_hqdn3d.Enable()
            self.rdb_nlmeans.SetSelection(0)
            self.rdb_nlmeans.Disable()
            self.rdb_hqdn3d.Disable()

    # ----------------------Event handler (callback)----------------------#

    def on_nlmeans(self, event):
        """
        Enable/disable Denoiser using `nlmeans` filter
        """
        if self.ckbx_nlmeans.IsChecked():
            self.rdb_nlmeans.Enable()
            self.rdb_hqdn3d.Disable()
            self.ckbx_hqdn3d.Disable()
            self.denoiser = "nlmeans"

        elif not self.ckbx_nlmeans.IsChecked():
            self.rdb_nlmeans.Disable()
            self.ckbx_hqdn3d.Enable()
            self.denoiser = ""
    # ------------------------------------------------------------------#

    def on_nlmeans_opt(self, event):
        """
        When `nlmeans` is enabled, even enables additional
        options on radiobox control.
        """
        opt = self.rdb_nlmeans.GetStringSelection()
        if opt == "Default":
            self.denoiser = "nlmeans"
        elif opt == "Old VHS tapes - good starting point restoration":
            self.denoiser = "nlmeans=8:3:2"
        elif opt == "Heavy - really noisy inputs":
            self.denoiser = "nlmeans=10:5:3"
        elif opt == "Light - good quality inputs":
            self.denoiser = "nlmeans=6:3:1"
    # ------------------------------------------------------------------#

    def on_hqdn3d(self, event):
        """
        Enable/disable Denoiser using `hqdn3d` filter
        """
        if self.ckbx_hqdn3d.IsChecked():
            self.ckbx_nlmeans.Disable()
            self.rdb_hqdn3d.Enable()
            self.denoiser = "hqdn3d"

        elif not self.ckbx_hqdn3d.IsChecked():
            self.ckbx_nlmeans.Enable()
            self.rdb_hqdn3d.Disable()
            self.denoiser = ""
    # ------------------------------------------------------------------#

    def on_hqdn3d_opt(self, event):
        """
        When `hqdn3d` is enabled, even enables additional
        options on radiobox control.
        """
        opt = self.rdb_hqdn3d.GetStringSelection()
        if opt == "Default":
            self.denoiser = "hqdn3d"

        elif opt == "Conservative [4.0:4.0:3.0:3.0]":
            self.denoiser = "hqdn3d=4.0:4.0:3.0:3.0"

        elif opt == "Old VHS tapes restoration [9.0:5.0:3.0:3.0]":
            self.denoiser = "hqdn3d=9.0:5.0:3.0:3.0"
    # ------------------------------------------------------------------#

    def on_help(self, event):
        """
        Open default web browser via Python Web-browser controller.
        see <https://docs.python.org/3.8/library/webbrowser.html>
        """
        page = ('https://jeanslack.github.io/Videomass/User-guide'
                '/Video_filters_en.pdf')

        webbrowser.open(page)
    # ------------------------------------------------------------------#

    def on_reset(self, event):
        """
        Reset all option and values
        """
        self.denoiser = ""  # deleting dictionary keys+values
        self.ckbx_nlmeans.SetValue(False)
        self.ckbx_hqdn3d.SetValue(False)
        self.ckbx_nlmeans.Enable()
        self.ckbx_hqdn3d.Enable()
        self.rdb_nlmeans.SetSelection(0)
        self.rdb_nlmeans.Disable()
        self.rdb_hqdn3d.SetSelection(0)
        self.rdb_hqdn3d.Disable()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Don't use self.Destroy() in this dialog
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See the caller for more info and usage.
        """
        return self.denoiser



# -*- coding: UTF-8 -*-
"""
Name: audioproperties.py
Porpose: A dialog interface for audio parameter settings
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.03.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx


class AudioProperties(wx.Dialog):
    """
    Provides a dialog for settings audio codec parameters
    which includes bit-rate, sample-rate, audio-channels
    and bit-per-sample (bit depth).

    See ``av_conversions.py`` -> ``audio_dialog`` method for
    how to use this class.
    """
    def __init__(self, parent, *args):
        """
        The given 'codecname' parameter represents the
        audio codec string, which will passed to the
        instance `audiodata`. This class has the same
        attributes as the AudioParameters class but
        here they are assigned by reference with the
        instance-object.

        """
        audiodata = AudioParameters(args[0])  # get audio params
        self.sample_rate = audiodata.sample_rate
        self.channels = audiodata.channels
        self.bitrate = audiodata.bitrate
        self.bitdepth = audiodata.bitdepth

        wx.Dialog.__init__(self, parent, -1, title=args[1],
                           style=wx.DEFAULT_DIALOG_STYLE
                           )
        sizerbase = wx.BoxSizer(wx.VERTICAL)
        boxctrls = wx.BoxSizer(wx.HORIZONTAL)
        sizerbase.Add(boxctrls, 0, wx.ALL | wx.CENTRE, 0)

        bitrate_list = [a[0] for a in self.bitrate.values()]
        self.rdb_bitrate = wx.RadioBox(self, wx.ID_ANY,
                                       ("Compression level"),
                                       choices=bitrate_list,
                                       majorDimension=0,
                                       style=wx.RA_SPECIFY_ROWS,
                                       )
        boxctrls.Add(self.rdb_bitrate, 0, wx.ALL, 5)

        channel_list = [a[0] for a in self.channels.values()]
        self.rdb_channels = wx.RadioBox(self, wx.ID_ANY,
                                        ("Channels"),
                                        choices=channel_list,
                                        majorDimension=0,
                                        style=wx.RA_SPECIFY_ROWS,
                                        )
        boxctrls.Add(self.rdb_channels, 0, wx.ALL, 5)

        samplerate_list = [a[0] for a in self.sample_rate.values()]
        self.rdb_sample_r = wx.RadioBox(self, wx.ID_ANY,
                                        ("Sample rate"),
                                        choices=samplerate_list,
                                        majorDimension=0,
                                        style=wx.RA_SPECIFY_ROWS,
                                        )
        boxctrls.Add(self.rdb_sample_r, 0, wx.ALL, 5)

        bitdepth_list = [a[0] for a in self.bitdepth.values()]
        self.rdb_bitdepth = wx.RadioBox(self, wx.ID_ANY,
                                        ("Bit per Sample (bit depth)"),
                                        choices=bitdepth_list,
                                        majorDimension=0,
                                        style=wx.RA_SPECIFY_ROWS,
                                        )
        boxctrls.Add(self.rdb_bitdepth, 0, wx.ALL, 5)

        # ---------------------- Bottom buttons ----------------------
        grid_btn = wx.GridSizer(1, 2, 0, 0)
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_reset = wx.Button(self, wx.ID_CLEAR, "")
        grid_btn.Add(btn_reset, 0, wx.ALL, 5)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        gridexit.Add(btn_ok, 0, wx.LEFT, 5)
        grid_btn.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 5)
        sizerbase.Add(grid_btn, 0, wx.EXPAND)
        # ---------------------- main layout ----------------------
        self.SetSizer(sizerbase)
        sizerbase.Fit(self)
        self.Layout()

        # ----------------------Properties----------------------
        if self.rdb_bitrate.GetStringSelection() == 'Not applicable ':
            self.rdb_bitrate.Hide()

        if self.rdb_bitdepth.GetStringSelection() == 'Not applicable ':
            self.rdb_bitdepth.Hide()

        if self.rdb_sample_r.GetStringSelection() == 'Not applicable ':
            self.rdb_sample_r.Hide()

        self.rdb_bitrate.SetToolTip(audiodata.BITRATE_TOOLTIP)
        self.rdb_channels.SetToolTip(audiodata.CHANNEL_TOOLTIP)
        self.rdb_sample_r.SetToolTip(audiodata.SAMPLE_RATE_TOOLTIP)
        self.rdb_bitdepth.SetToolTip(audiodata.BITDEPTH_TOOLTIP)

        # Set previusly settings:
        arate, adepth, abitrate, achannel = args[2], args[3], args[4], args[5]
        if arate[0]:
            self.rdb_sample_r.SetSelection(samplerate_list.index(arate[0]))
        else:
            self.rdb_sample_r.SetSelection(0)
        if adepth[0]:
            self.rdb_bitdepth.SetSelection(bitdepth_list.index(adepth[0]))
        else:
            self.rdb_bitdepth.SetSelection(0)
        if abitrate[0]:
            self.rdb_bitrate.SetSelection(bitrate_list.index(abitrate[0]))
        else:
            self.rdb_bitrate.SetSelection(0)
        if achannel[0]:
            self.rdb_channels.SetSelection(channel_list.index(achannel[0]))
        else:
            self.rdb_channels.SetSelection(0)

        # --------------------Binders (EVT)----------------------
        self.Bind(wx.EVT_BUTTON, self.on_cancel, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_apply, btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)

    # ------------------------------------------------------------------#

    def on_reset(self, event):
        """
        Reset all option and values
        """
        self.rdb_sample_r.SetSelection(0)
        self.rdb_bitdepth.SetSelection(0)
        self.rdb_bitrate.SetSelection(0)
        self.rdb_channels.SetSelection(0)
    # ------------------------------------------------------------------#

    def on_cancel(self, event):
        """
        Cancel this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_apply(self, event):
        """
        Don't use self.Destroy() in this dialog
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See the caller for more info and usage.

        Returns:
            A four items tuple with audio parameters:
                channel(str(description), str(ffmpeg flags))
                sample_rate(str(description), str(ffmpeg flags))
                bitrate(str(description), str(ffmpeg flags))
                bitdepth(str(description), str(ffmpeg flags))
        """
        sel_ch = self.rdb_channels.GetStringSelection()
        sel_sr = self.rdb_sample_r.GetStringSelection()
        sel_br = self.rdb_bitrate.GetStringSelection()
        sel_bd = self.rdb_bitdepth.GetStringSelection()

        return ([v for v in self.channels.values() if sel_ch in v[0]][0],
                [v for v in self.sample_rate.values() if sel_sr in v[0]][0],
                [v for v in self.bitrate.values() if sel_br in v[0]][0],
                [v for v in self.bitdepth.values() if sel_bd in v[0]][0],
                )
# ----------------------------------------------------------------------#


class AudioParameters():
    """
    It provides an adequate representation of the adjustment
    parameters related to some audio codecs as bitrate, sample
    rate, channels and bit depth based on FFmpeg syntax.
    For a list of supported audio codecs, see the
    __init__ method docstring.

    USAGE:
        >>> adata = AudioParameters(str(codec))
        >>> if not adata.unsupported:
        >>>    sample_rate = adata.sample_rate
        >>>    channels = adata.channels
        >>>    bitrate = adata.bitrate
        >>>    bitdepth = adata.bitdepth
    EXAMPLES:
        >>>    descriptions = [a[0] for a in adata.sample_rate.values()]
        >>>    ffmpeg_flags = [a[1] for a in adata.sample_rate.values()]

    """
    CHANNEL_TOOLTIP = (_('Videomass supports mono and stereo audio channels. '
                         'If you are not sure set to "Auto" and source values '
                         'will be copied.'
                         ))
    SAMPLE_RATE_TOOLTIP = (_('The audio Rate (or sample-rate) is the sound '
                             'sampling frequency and is measured in Hertz. '
                             'The higher the frequency, the more true it '
                             'will be to the sound source and the more the '
                             'file will increase in size. For audio CD '
                             'playback, set a sampling frequency of 44100 '
                             'kHz. If you are not sure, set to "Auto" and '
                             'source values will be copied.'
                             ))
    BITRATE_TOOLTIP = (_('The audio bitrate affects the file compression and '
                         'thus the quality of listening. The higher the '
                         'value, the higher the quality.'
                         ))
    BITDEPTH_TOOLTIP = (_('Bit depth is the number of bits of information in '
                          'each sample, and it directly corresponds to the '
                          'resolution of each sample. Bit depth is only '
                          'meaningful in reference to a PCM digital signal. '
                          'Non-PCM formats, such as lossy compression '
                          'formats, do not have associated bit depths.'
                          ))
    SAMPLE_RATE = {0: ("Auto", ""),
                   1: ("44100 Hz ", "-ar 44100"),
                   2: ("48000 Hz ", "-ar 48000"),
                   3: ("88200 Hz ", "-ar 88200"),
                   4: ("96000 Hz ", "-ar 96000"),
                   5: ("192000 Hz ", "-ar 192000"),
                   }
    # ----------------------------------------------------------------#

    def __init__(self, acodecname: str):
        """
        Expects a type string object representing the audio
        codec name. The following audio codec are supported:

        'PCM', 'FLAC', 'ALAC', 'AAC', 'AC3', 'VORBIS', 'LAME', 'OPUS

        If any of the audio codecs described above are passed,
        this class sets all instance attributes with appropriate
        class method.
        """
        self.sample_rate = {0: ('Not applicable ', "")}
        self.channels = {0: ('Not applicable ', "")}
        self.bitrate = {0: ('Not applicable ', "")}
        self.bitdepth = {0: ('Not applicable ', "")}
        self.unsupported = None

        if acodecname in ('PCM', 'wav', 'aiff'):
            self.pcm()
        elif acodecname in ('FLAC', 'flac'):
            self.flac()
        elif acodecname in ('ALAC', 'alac', 'm4a'):
            self.alac()
        elif acodecname in ('AAC', 'aac'):
            self.aac()
        elif acodecname in ('AC3', 'ac3'):
            self.ac3()
        elif acodecname in ('VORBIS', 'ogg', 'oga'):
            self.vorbis()
        elif acodecname in ('LAME', 'mp3'):
            self.lame()
        elif acodecname in ('OPUS', 'opus'):
            self.opus()
        else:
            self.unsupported = f'Unsupported codec: {acodecname}'
    # -----------------------------------------------------------------#

    def pcm(self):
        """
        Set the PCM codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
        self.bitdepth = {0: ("Auto", ""),
                         1: ("16 bit", "pcm_s16le"),
                         2: ("24 bit", "pcm_s24le"),
                         4: ("32 bit", "pcm_s32le"),
                         }
    # -----------------------------------------------------------------#

    def flac(self):
        """
        Set the FLAC codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("very high quality", "-compression_level 0"),
                        2: ("quality 1", "-compression_level 1"),
                        3: ("quality 2", "-compression_level 2"),
                        4: ("quality 3", "-compression_level 3"),
                        5: ("quality 4", "-compression_level 4"),
                        6: ("Default quality", "-compression_level 5"),
                        7: ("quality 6", "-compression_level 6"),
                        8: ("quality 7", "-compression_level 7"),
                        9: ("low quality", "-compression_level 8"),
                        }
    # -----------------------------------------------------------------#

    def alac(self):
        """
        Set the ALAC codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
    # -----------------------------------------------------------------#

    def opus(self):
        """
        Set the OPUS codec parameters
        """
        # self.sample_rate
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("low quality 0", "-compression_level 0"),
                        2: ("low quality 1", "-compression_level 1"),
                        3: ("quality 2", "-compression_level 2"),
                        4: ("quality 3", "-compression_level 3"),
                        5: ("quality 4", "-compression_level 4"),
                        6: ("medium quality 5", "-compression_level 5"),
                        7: ("quality 6", "-compression_level 6"),
                        8: ("quality 7", "-compression_level 7"),
                        9: ("quality 8", "-compression_level 8"),
                        10: ("high quality 9", "-compression_level 9"),
                        11: ("highest quality 10 (default)",
                             "-compression_level 10"),
                        }
    # -----------------------------------------------------------------#

    def aac(self):
        """
        Set the AAC codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         3: ("MultiChannel 5.1", "-ac 6"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("low quality", "-b:a 128k"),
                        2: ("medium/low quality", "-b:a 160k"),
                        3: ("medium quality", "-b:a 192k"),
                        4: ("good quality", "-b:a 260k"),
                        5: ("very good quality", "-b:a 320k"),
                        }
    # -----------------------------------------------------------------#

    def ac3(self):
        """
        Set the AC3 codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         3: ("MultiChannel 5.1", "-ac 6"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("low quality", "-b:a 192k"),
                        2: ("224 kbit/s", "-b:a 224k"),
                        3: ("256 kbit/s", "-b:a 256k"),
                        4: ("320 kbit/s", "-b:a 320k"),
                        5: ("384 kbit/s", "-b:a 384k"),
                        6: ("448 kbit/s", "-b:a 448k"),
                        7: ("512 kbit/s", "-b:a 512k"),
                        8: ("576 kbit/s", "-b:a 576k"),
                        9: ("very good quality", "-b:a 640k"),
                        }
    # -----------------------------------------------------------------#

    def vorbis(self):
        """
        Set the VORBIS codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("very poor quality", "-aq 1"),
                        2: ("VBR 92 kbit/s", "-aq 2"),
                        3: ("VBR 128 kbit/s", "-aq 3"),
                        4: ("VBR 160 kbit/s", "-aq 4"),
                        5: ("VBR 175 kbit/s", "-aq 5"),
                        6: ("VBR 192 kbit/s", "-aq 6"),
                        7: ("VBR 220 kbit/s", "-aq 7"),
                        8: ("VBR 260 kbit/s", "-aq 8"),
                        9: ("VBR 320 kbit/s", "-aq 9"),
                        10: ("very good quality", "-aq 10"),
                        }
    # -----------------------------------------------------------------#

    def lame(self):
        """
        Set the LAME codec parameters
        """
        self.sample_rate = AudioParameters.SAMPLE_RATE
        self.channels = {0: ("Auto", ""),
                         1: ("Mono", "-ac 1"),
                         2: ("Stereo", "-ac 2"),
                         }
        self.bitrate = {0: ("Auto", ""),
                        1: ("VBR 128 kbit/s (low quality)", "-b:a 128k"),
                        2: ("VBR 160 kbit/s", "-b:a 160k"),
                        3: ("VBR 192 kbit/s", "-b:a 192k"),
                        4: ("VBR 260 kbit/s", "-b:a 260k"),
                        5: ("CBR 320 kbit/s (very good quality)", "-b:a 320k"),
                        }



# -*- coding: UTF-8 -*-
"""
Name: playlist_indexing.py
Porpose: shows a dialog box for setting playlist indexing
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import re
import wx
import wx.lib.mixins.listctrl as listmix


class ListCtrl(wx.ListCtrl,
               listmix.ListCtrlAutoWidthMixin,
               listmix.TextEditMixin):
    """
    A listctrl with the ability to be editable.
    """

    def __init__(self, parent, ID, pos=wx.DefaultPosition,
                 size=wx.DefaultSize, style=0):
        wx.ListCtrl.__init__(self, parent, ID, pos, size, style)

        listmix.ListCtrlAutoWidthMixin.__init__(self)
        self.populate()
        listmix.TextEditMixin.__init__(self)

    def populate(self):
        """populate with default colums"""
        self.InsertColumn(0, '#', width=30)
        self.InsertColumn(1, _('URL'), width=400)
        self.InsertColumn(2, _('Playlist Items'), width=200)


class Indexing(wx.Dialog):
    """
    Shows a dialog box for setting playlist indexing.
    See ``youtubedl_ui.py`` -> ``on_playlist_idx`` method for
    how to use this class.

    """
    get = wx.GetApp()  # get data from bootstrap
    OS = get.appset['ostype']
    appdata = get.appset
    if appdata['IS_DARK_THEME'] is True:
        GREEN = '#136322'
    elif appdata['IS_DARK_THEME'] is False:
        GREEN = '#4CDD67'
    else:
        GREEN = '#40804C'
    appicon = get.iconset['videomass']

    HELPME = (_('To index the media of a playlist, click on the "Playlist '
                'Items" column of each corresponding URL and specify the '
                'numerical indexes separated by commas, e.g. "1,2,5,8" if '
                'you want to download the indexed media at 1, 2, 5, 8 of the '
                'playlist.\nIt is also possible to specify intervals, e.g. '
                '"1-3,7,10-13" with which the media at index 1, 2, 3, 7, 10, '
                '11, 12 and 13 will be downloaded.\n'))

    def __init__(self, parent, url, data):
        """
        NOTE Use 'parent, -1' param. to make parent, use 'None' otherwise

        """
        self.clrs = Indexing.appdata['colorscheme']
        self.urls = url
        self.data = data

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)

        # ------ Add widget controls
        self.lctrl = ListCtrl(self,
                              wx.ID_ANY,
                              style=wx.LC_REPORT
                              | wx.SUNKEN_BORDER
                              | wx.LC_SINGLE_SEL
                              | wx.LC_HRULES
                              | wx.LC_VRULES
                              )
        self.tctrl = wx.TextCtrl(self,
                                 wx.ID_ANY, "",
                                 style=wx.TE_MULTILINE
                                 | wx.TE_READONLY
                                 | wx.TE_RICH2
                                 )

        # ------ Properties
        self.SetTitle(_('Playlist Editor'))
        self.SetMinSize((800, 400))
        self.lctrl.SetMinSize((800, 200))
        self.tctrl.SetMinSize((800, 200))

        # ------ set Layout
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.lctrl, 0, wx.ALL | wx.EXPAND, 5)

        labtstr = _('Help viewer')
        lab = wx.StaticText(self, label=labtstr)
        sizer_1.Add(lab, 0, wx.LEFT, 5)

        grid_sizer_1 = wx.GridSizer(1, 1, 0, 0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)
        grid_sizer_1.Add(self.tctrl, 0, wx.ALL | wx.EXPAND, 5)

        # ------ bottom layout for buttons
        grid_btn = wx.GridSizer(1, 2, 0, 0)
        gridexit = wx.BoxSizer(wx.HORIZONTAL)
        btn_reset = wx.Button(self, wx.ID_CLEAR, "")
        grid_btn.Add(btn_reset, 0, wx.ALL, 5)
        btn_cancel = wx.Button(self, wx.ID_CANCEL, "")
        gridexit.Add(btn_cancel, 0)
        btn_ok = wx.Button(self, wx.ID_OK)
        gridexit.Add(btn_ok, 0, wx.LEFT, 5)
        grid_btn.Add(gridexit, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, 5)

        # ------ final settings:
        sizer_1.Add(grid_btn, 0, wx.EXPAND)
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(Indexing.appicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()

        index = 0

        for link in url:
            self.lctrl.InsertItem(index, str(index + 1))
            self.lctrl.SetItem(index, 1, link)
            if '/playlist' in link:
                self.lctrl.SetItemBackgroundColour(index, Indexing.GREEN)

            if not self.data == {'': ''}:
                for key, val in self.data.items():
                    if key == link:
                        self.lctrl.SetItem(index, 2, val)
            index += 1

        if Indexing.OS == 'Darwin':
            self.lctrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
            self.tctrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
        else:
            self.lctrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            self.tctrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
            lab.SetLabelMarkup(f"<b>{labtstr}</b>")

        self.tctrl.SetBackgroundColour(self.clrs['BACKGRD'])

        # ----------------------Binding (EVT)----------------------#
        self.lctrl.Bind(wx.EVT_LIST_BEGIN_LABEL_EDIT, self.on_edit_begin)
        self.lctrl.Bind(wx.EVT_LIST_END_LABEL_EDIT, self.on_edit_end)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_cancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btn_ok)
        self.Bind(wx.EVT_BUTTON, self.on_reset, btn_reset)

        self.textstyle()

    def textstyle(self):
        """
        clear log messages and set text style on textctrl box
        """
        self.tctrl.Clear()
        self.tctrl.SetDefaultStyle(wx.TextAttr(self.clrs['TXT1']))
        self.tctrl.AppendText(f'{Indexing.HELPME}')
    # ------------------------------------------------------------------#

    def getvalue(self):
        """
        This method return values via the interface getvalue()
        by the caller. See the caller for more info and usage.
        """
        diz = {}
        for row, url in enumerate(self.urls):
            txt = self.lctrl.GetItem(row, 2).GetText()
            if txt:
                diz[url] = ''.join(txt.split())
        return diz

    # ----------------------Event handler (callback)----------------------#

    def on_edit_end(self, event):
        """
        Checking event-entered strings using REGEX:

            Allows min 0 to max 999 digits and does not allow
            numbers with leading zeroes like 07 or 005, allows
            a number separated by hyphen like 22-33 and supports
            comma followed by a white space. Note that all white
            spaces are stripped before come to parsing by REGEX:
            see `string` var below.

        Some event examples:

        row_id = event.GetIndex()  # Get the current row
        col_id = event.GetColumn()  # Get the current column
        new_data = event.GetLabel()  # Get the changed data
        cols = self.lctrl.GetColumnCount()  # Get the total number of col
        rows = self.lctrl.GetItemCount()  # Get the total number of rows

        """
        wxd = wx.DateTime.Now()
        date = wxd.Format('%H:%M:%S')
        new_data = event.GetLabel()  # Get the changed data
        errbeg = _('ERROR: Invalid option')
        errend = _('please try again.')
        assign = _('OK: Indexes to download')
        string = ''.join(new_data.split())

        if string == '':
            event.Veto()
            return
        check = bool(re.search(r"^(?:[1-9]\d\d|[1-9]?\d)(?:-(?:[1-9]\d\d"
                               r"|[1-9]?\d))?(?:,\s?(?:[1-9]\d\d|[1-9]?\d)"
                               r"(?:-(?:[1-9]\d\d|[1-9]?\d))?)*$", string))
        if check is not True:
            self.tctrl.SetDefaultStyle(wx.TextAttr(self.clrs['ERR1']))
            self.tctrl.AppendText(f'\n{date}: {errbeg}: '
                                  f'"{new_data}", {errend}\n')
            event.Veto()
            return

        self.tctrl.SetDefaultStyle(wx.TextAttr(self.clrs['TXT3']))
        self.tctrl.AppendText(f'\n{date}: {assign}: "{string}"\n')
    # ------------------------------------------------------------------#

    def on_edit_begin(self, event):
        """
        Columns 0 and 1 must not be editable for
        link without playlist.
        """
        row_id = event.GetIndex()

        wxd = wx.DateTime.Now()
        date = wxd.Format('%H:%M:%S')
        invalidmsg = _('WARNING: The selected URL does not refer to a '
                       'playlist. Only lines marked green can be indexed.')

        colour = Indexing.GREEN

        if event.GetColumn() in (0, 1):
            event.Veto()
        elif event.GetColumn() == 2:
            # It looks like the HTML color codes are translated to RGB here
            if self.lctrl.GetItemBackgroundColour(row_id) != colour:
                self.tctrl.SetDefaultStyle(wx.TextAttr(self.clrs['WARN']))
                self.tctrl.AppendText(f'\n{date}: {invalidmsg}\n')
                event.Veto()
            else:
                event.Skip()  # or event.Allow()
        else:
            event.Skip()  # or event.Allow()
    # ------------------------------------------------------------------#

    def on_reset(self, event):
        """
        Reset all items on editable columns and clear log messages
        """
        rows = self.lctrl.GetItemCount()  # Get the total number of rows
        for row in range(rows):
            self.lctrl.SetItem(row, 2, '')

        self.textstyle()
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Close this dialog without saving anything
        """
        event.Skip()
    # ------------------------------------------------------------------#

    def on_ok(self, event):
        """
        Don't use self.Destroy() in this dialog
        """
        event.Skip()



# -*- coding: UTF-8 -*-
"""
Name: configurator.py
Porpose: Set Videomass configuration on startup
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.23.2024
Code checker: flake8, pylint

 This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import site
import shutil
import platform
from videomass.vdms_utils.utils import copydir_recursively
from videomass.vdms_sys.settings_manager import ConfigManager


def msg(arg):
    """
    print logging messages during startup
    """
    print('Info:', arg)


def create_dirs(dirname, fconf):
    """
    This function is responsible for the recursive creation
    of directories required for Videomass if they do not exist.
    Returns dict:
        key == 'R'
        key == ERROR (if any errors)
    """
    if not os.path.exists(dirname):
        try:
            os.makedirs(dirname, mode=0o777)
        except FileExistsError as err:
            return {'ERROR': err}
        except OSError as err:
            os.remove(fconf)  # force to restart on deleting
            thismsg = ('Please try restarting Videomass to '
                       'restore default settings now.')
            return {'ERROR': f'{err}\n{thismsg}'}

    return {'R': None}


def restore_presets_dir(dirconf, srcpath):
    """
    Restore preset directory from source if it doesn't exist.
    Returns dict:
        key == 'R'
        key == ERROR (if any errors)
    """
    if not os.path.exists(os.path.join(dirconf, "presets")):
        # try to restoring presets directory on videomass dir
        drest = copydir_recursively(os.path.join(srcpath, "presets"), dirconf)
        if drest:
            return {'ERROR': drest}

    return {'R': None}


def get_options(dirconf, fileconf, srcpath, makeportable):
    """
    Check the application options. Reads the `settings.json`
    file; if it does not exist or is unreadable try to restore
    it. If `dirconf` does not exist try to restore both `dirconf`
    and `settings.json`. If VERSION is not the same as the version
    readed, it adds new missing items while preserving the old ones
    with the same values.

    Returns dict:
        key == 'R'
        key == ERROR (if any errors)
    """
    conf = ConfigManager(fileconf, makeportable)
    version = ConfigManager.VERSION

    if os.path.exists(dirconf):  # i.e ~/.conf/videomass dir
        if os.path.isfile(fileconf):
            data = {'R': conf.read_options()}
            if not data['R']:
                conf.write_options()
                data = {'R': conf.read_options()}
            if float(data['R']['confversion']) != version:  # conf version
                data['R']['confversion'] = version
                new = ConfigManager.DEFAULT_OPTIONS  # model
                data = {'R': {**new, **data['R']}}
                conf.write_options(**data['R'])
        else:
            conf.write_options()
            data = {'R': conf.read_options()}

    else:  # try to restore entire configuration directory
        dconf = copydir_recursively(srcpath,
                                    os.path.dirname(dirconf),
                                    "videomass",
                                    )
        if dconf:
            data = {'ERROR': dconf}
        else:
            conf.write_options()
            data = {'R': conf.read_options()}

    return data


def get_pyinstaller():
    """
    Get pyinstaller-based package attributes to determine
    how to use sys.executable
    When a bundled app starts up, the bootloader sets the sys.frozen
    attribute and stores the absolute path to the bundle folder
    in sys._MEIPASS.
    For a one-folder bundle, this is the path to that folder.
    For a one-file bundle, this is the path to the temporary folder
    created by the bootloader.
    """
    if getattr(sys, 'frozen', False) and hasattr(sys, '_MEIPASS'):
        frozen, meipass = True, True
        this = getattr(sys, '_MEIPASS', os.path.abspath(__file__))
        data_locat = this

    else:
        frozen, meipass = False, False
        this = os.path.realpath(os.path.abspath(__file__))
        data_locat = os.path.dirname(os.path.dirname(this))

    return frozen, meipass, this, data_locat


def conventional_paths():
    """
    Establish the conventional paths based on OS

    """
    user_name = os.path.expanduser('~')

    if platform.system() == 'Windows':
        fpath = "\\AppData\\Roaming\\videomass\\settings.json"
        file_conf = os.path.join(user_name + fpath)
        dir_conf = os.path.join(user_name + "\\AppData\\Roaming\\videomass")
        log_dir = os.path.join(dir_conf, 'log')  # logs
        cache_dir = os.path.join(dir_conf, 'cache')  # updates executable
        trash_dir = os.path.join(dir_conf, "Trash")

    elif platform.system() == "Darwin":
        fpath = "Library/Application Support/videomass/settings.json"
        file_conf = os.path.join(user_name, fpath)
        dir_conf = os.path.join(user_name, os.path.dirname(fpath))
        log_dir = os.path.join(user_name, "Library/Logs/videomass")
        cache_dir = os.path.join(user_name, "Library/Caches/videomass")
        trash_dir = os.path.join(dir_conf, "Trash")

    else:  # Linux, FreeBsd, etc.
        fpath = ".config/videomass/settings.json"
        file_conf = os.path.join(user_name, fpath)
        dir_conf = os.path.join(user_name, ".config/videomass")
        log_dir = os.path.join(user_name, ".local/share/videomass/log")
        cache_dir = os.path.join(user_name, ".cache/videomass")
        trash_dir = os.path.join(dir_conf, "Trash")

    return file_conf, dir_conf, log_dir, cache_dir, trash_dir


def portable_paths(portdirname):
    """
    Make portable-data paths based on OS

    """
    dir_conf = portdirname
    file_conf = os.path.join(dir_conf, "settings.json")
    log_dir = os.path.join(dir_conf, 'log')  # logs
    cache_dir = os.path.join(dir_conf, 'cache')  # updates executable
    trash_dir = os.path.join(dir_conf, "Trash")

    if not os.path.exists(dir_conf):
        os.makedirs(dir_conf, mode=0o777)

    return file_conf, dir_conf, log_dir, cache_dir, trash_dir


def get_color_scheme(theme):
    """
    Returns the corrisponding colour scheme according to
    chosen theme in ("Videomass-Light",
                     "Videomass-Dark",
                     "Videomass-Colours",
                     "Ubuntu-Light-Aubergine",
                     "Ubuntu-Dark-Aubergine",
                     )
    """
    if theme == 'Videomass-Colours':
        c_scheme = {'BACKGRD': '#8dc6c2',  # Ciano: background color
                    'TXT0': '#d01d7a',  # Magenta: titles & end messages)
                    'TXT1': '#ae11db',  # Purple for debug and others
                    'ERR0': '#FF4A1B',  # ORANGE: errors 1 and others
                    'WARN': '#7b6e01',  # YELLOW: warnings
                    'ERR1': '#EA312D',  # LIGHTRED: errors 2
                    'SUCCESS': '#017001',  # Light GREEN: successful
                    'TXT3': '#333333',  # Dark grey: standard text
                    'INFO': '#194c7e',  # Blue: other info messages
                    'DEBUG': '#333333',  # light green
                    'FAILED': '#D21814',  # RED_DEEP if failed
                    'ABORT': '#A41EA4',  # RED_DEEP if abort
                    }
    elif theme == 'Videomass-Dark':
        c_scheme = {'BACKGRD': '#232424',  # DARK Grey background color
                    'TXT0': '#FFFFFF',  # WHITE for title or URL in progress
                    'TXT1': '#959595',  # GREY for all other text messages
                    'ERR0': '#FF4A1B',  # ORANGE for error text messages
                    'WARN': '#dfb72f',  # YELLOW for warning text messages
                    'ERR1': '#EA312D',  # LIGHTRED for errors 2
                    'SUCCESS': '#30ec30',  # Light GREEN when it is successful
                    'TXT3': '#11b584',  # medium green
                    'INFO': '#118db5',  # AZURE
                    'DEBUG': '#0ce3ac',  # light green
                    'FAILED': '#D21814',  # RED_DEEP if failed
                    'ABORT': '#A41EA4',  # VIOLET the user stops the processes
                    }
    elif theme == 'Videomass-Light':
        c_scheme = {'BACKGRD': '#ced0d1',  # WHITE background color
                    'TXT0': '#1f1f1f',  # BLACK for title or URL in progress
                    'TXT1': '#576470',  # LIGHT_SLATE for all other text msg
                    'ERR0': '#d25c07',  # ORANGE for error text messages
                    'WARN': '#988313',  # YELLOW for warning text messages
                    'ERR1': '#c8120b',  # LIGHTRED for errors 2
                    'SUCCESS': '#35a735',  # DARK_GREEN when it is successful
                    'TXT3': '#005c00',  # Light Green
                    'INFO': '#3298FB',  # AZURE
                    'DEBUG': '#005c00',  # Light Green
                    'FAILED': '#D21814',  # RED_DEEP if failed
                    'ABORT': '#A41EA4',  # VIOLET the user stops the processes
                    }
    elif theme in ('Ubuntu-Dark-Aubergine', 'Ubuntu-Light-Aubergine'):
        c_scheme = {'BACKGRD': '#2C001E',  # Dark-Aubergine background color
                    'TXT0': '#FFFFFF',  # WHITE for titles
                    'TXT1': '#8AB8E6',  # light Blue
                    'ERR0': '#E95420',  # ORANGE for error text messages
                    'WARN': '#dfb72f',  # YELLOW for warning messages
                    'ERR1': '#F90808',  # RED_DEEP
                    'SUCCESS': '#ABD533',  # Light GREEN when it is successful
                    'TXT3': '#AEA79F',  # Ubuntu warm grey (base foreground)
                    'INFO': '#F7C3B1',  # Ubuntu orange light 30%
                    'DEBUG': '#8AB8E6',  # light Blue
                    'FAILED': '#F90808',  # RED_DEEP
                    'ABORT': '#F90808',  # RED_DEEP
                    }
    else:
        c_scheme = {'ERROR': f'Unknow theme "{theme}"'}

    return c_scheme


def data_location(args):
    """
    Determines data location and modes to make the app
    portable, fully portable or using conventional paths.
    return data location.
    """
    frozen, meipass, this, locat = get_pyinstaller()
    workdir = os.path.dirname(os.path.dirname(os.path.dirname(this)))

    if args['make_portable']:
        portdir = args['make_portable']
        (conffile, confdir, logdir,
         cachedir, trash_dir) = portable_paths(portdir)
    else:
        conffile, confdir, logdir, cachedir, trash_dir = conventional_paths()

    return {"conffile": conffile,
            "confdir": confdir,
            "logdir": logdir,
            "cachedir": cachedir,
            "trashdir_default": trash_dir,
            "this": this,
            "frozen": frozen,
            "meipass": meipass,
            "locat": locat,
            "localepath": os.path.join(locat, 'locale'),
            "workdir": workdir,
            "srcpath": os.path.join(locat, 'share'),
            "icodir": os.path.join(locat, 'art', 'icons'),
            "ffmpeg_pkg": os.path.join(locat, 'FFMPEG'),
            }


class DataSource():
    """
    DataSource class determines the Videomass's configuration
    according to the used Operating System (Linux/*BSD, MacOS, Windows).
    Remember that all specifications defined in this class refer
    only to the following packaging methods:

        - Videomass Python package by PyPi (cross-platform, multiuser/user)
        - Linux's distributions packaging (multiuser)
        - Bundled app by pyinstaller (windowed for MacOs and Windows)
        - AppImage for Linux (user)

        * multiuser: system installation
        * user: local installation

    """
    # -------------------------------------------------------------------

    def __init__(self, kwargs):
        """
        Having the pathnames returned by `dataloc` it
        performs the initialization described in DataSource.

        """
        self.dataloc = data_location(kwargs)
        self.relativepaths = bool(kwargs['make_portable'])
        self.makeportable = kwargs['make_portable']
        self.apptype = None  # appimage, pyinstaller on None
        launcher = os.path.isfile(f"{self.dataloc['workdir']}/launcher")

        if self.dataloc['frozen'] and self.dataloc['meipass'] or launcher:
            msg(f"frozen={self.dataloc['frozen']} "
                f"meipass={self.dataloc['meipass']} "
                f"launcher={launcher}")

            self.apptype = 'pyinstaller' if not launcher else None
            self.prg_icon = f"{self.dataloc['icodir']}/videomass.png"

        elif ('/tmp/.mount_' in sys.executable or os.path.exists(
              os.path.dirname(os.path.dirname(os.path.dirname(
                  sys.argv[0])))
              + '/AppRun')):
            # embedded on python appimage
            msg('Embedded on python appimage')
            self.apptype = 'appimage'
            userbase = os.path.dirname(os.path.dirname(sys.argv[0]))
            pixmaps = '/share/pixmaps/videomass.png'
            self.prg_icon = os.path.join(userbase + pixmaps)

        else:
            binarypath = shutil.which('videomass')
            if platform.system() == 'Windows':  # any other packages
                exe = binarypath if binarypath else sys.executable
                msg(f'Win32 executable={exe}')
                # HACK check this
                # dirname = os.path.dirname(sys.executable)
                # pythonpath = os.path.join(dirname, 'Script', 'videomass')
                # self.icodir = dirname + '\\share\\videomass\\icons'
                self.prg_icon = self.dataloc['icodir'] + "\\videomass.png"

            elif binarypath == '/usr/local/bin/videomass':
                msg(f'executable={binarypath}')
                # pip as super user, usually Linux, MacOs, Unix
                share = '/usr/local/share/pixmaps'
                self.prg_icon = share + '/videomass.png'

            elif binarypath == '/usr/bin/videomass':
                msg(f'executable={binarypath}')
                # installed via apt, rpm, etc, usually Linux
                share = '/usr/share/pixmaps'
                self.prg_icon = share + "/videomass.png"

            else:
                msg(f'executable={binarypath}')
                # pip as user, usually Linux, MacOs, Unix
                if binarypath is None:
                    # need if user $PATH is not set yet
                    userbase = site.getuserbase()
                else:
                    userbase = os.path.dirname(os.path.dirname(binarypath))
                pixmaps = '/share/pixmaps/videomass.png'
                self.prg_icon = os.path.join(userbase + pixmaps)
    # ---------------------------------------------------------------------

    def get_fileconf(self):
        """
        Get settings.json configuration data and returns a dict object
        with current data-set for bootstrap.

        Note: If returns a dict key == ERROR it will raise a windowed
        fatal error in the gui_app bootstrap.
        """
        # handle configuration file
        userconf = get_options(self.dataloc['confdir'],
                               self.dataloc['conffile'],
                               self.dataloc['srcpath'],
                               self.makeportable,
                               )
        if userconf.get('ERROR'):
            return userconf
        userconf = userconf['R']

        # restore presets folder
        presets_rest = restore_presets_dir(self.dataloc['confdir'],
                                           self.dataloc['srcpath'],
                                           )
        if presets_rest.get('ERROR'):
            return presets_rest

        # create the required directories if not existing
        requiredirs = (os.path.join(self.dataloc['cachedir'], 'tmp'),
                       self.dataloc['logdir'],
                       userconf['outputdir'],
                       userconf['ydlp-outputdir'],
                       self.dataloc['trashdir_default']
                       )
        if not userconf['trashdir_loc'].strip():
            userconf['trashdir_loc'] = self.dataloc['trashdir_default']
        for dirs in requiredirs:
            create = create_dirs(dirs, self.dataloc['conffile'],)
            if create.get('ERROR'):
                return create

        # set color scheme
        theme = get_color_scheme(userconf['icontheme'])
        userconf['colorscheme'] = theme
        if theme.get('ERROR'):
            return theme

        def _relativize(path, relative=self.relativepaths):
            """
            Returns a relative pathname if *relative* param is True.
            If not, it returns the given pathname. Also return the given
            pathname if `ValueError` is raised. This function is called
            several times during program execution.
            """
            try:
                return os.path.relpath(path) if relative else path
            except (ValueError, TypeError):
                # return {'ERROR': f'{error}'}  # use `as error` here
                return path

        return ({'ostype': platform.system(),
                 'srcpath': _relativize(self.dataloc['srcpath']),
                 'localepath': _relativize(self.dataloc['localepath']),
                 'fileconfpath': _relativize(self.dataloc['conffile']),
                 'workdir': _relativize(self.dataloc['workdir']),
                 'confdir': _relativize(self.dataloc['confdir']),
                 'logdir': _relativize(self.dataloc['logdir']),
                 'cachedir': _relativize(self.dataloc['cachedir']),
                 'trashdir_default':
                     _relativize(self.dataloc['trashdir_default']),
                 'FFMPEG_videomass_pkg':
                     _relativize(self.dataloc['ffmpeg_pkg']),
                 'app': self.apptype,
                 'relpath': self.relativepaths,
                 'getpath': _relativize,
                 'ffmpeg_cmd': _relativize(userconf['ffmpeg_cmd']),
                 'ffprobe_cmd': _relativize(userconf['ffprobe_cmd']),
                 'ffplay_cmd': _relativize(userconf['ffplay_cmd']),
                 'ffmpeg-default-args': '-y -stats -hide_banner',
                 'ffplay-default-args': '-hide_banner',
                 **userconf
                 })
    # --------------------------------------------------------------------

    def icons_set(self, icontheme):
        """
        Determines icons set assignment defined on the configuration
        file (see `Set icon themes map:`, on paragraph `6- GUI setup`
        in the settings.json file).
        Returns a icontheme dict object.

        """
        keys = ('videomass', 'A/V-Conv', 'startconv', 'fileproperties',
                'playback', 'concatenate', 'preview', 'clear',
                'addtoprst', 'scale', 'crop', 'rotate', 'deinterlace',
                'denoiser', 'volanalyze', 'settings', 'audiovolume',
                'presets_manager', 'profile_add', 'profile_del',
                'profile_edit', 'previous', 'next', 'stabilizer',
                'preview_audio', 'profile_copy', 'slideshow',
                'videotopictures', 'atrack', 'timerset', 'coloreq',
                'stop', 'home', 'youtube', 'playlist',
                'download', 'statistics', 'play', 'subtitles',
                'proc-queue', 'add-queue', 'options',
                )  # must match with items on `iconset` tuple, see following

        icodir = self.dataloc['icodir']
        iconames = {'Videomass-Light':  # Videomass icons for light themes
                    {'x48': f'{icodir}/Sign_Icons/48x48_light',
                     'x16': f'{icodir}/Videomass-Light/16x16',
                     'x22': f'{icodir}/Videomass-Light/24x24'},
                    'Videomass-Dark':  # Videomass icons for dark themes
                    {'x48': f'{icodir}/Sign_Icons/48x48_dark',
                     'x16': f'{icodir}/Videomass-Dark/16x16',
                     'x22': f'{icodir}/Videomass-Dark/24x24'},
                    'Videomass-Colours':  # Videomass icons for all themes
                    {'x48': f'{icodir}/Sign_Icons/48x48',
                     'x16': f'{icodir}/Videomass-Colours/16x16',
                     'x22': f'{icodir}/Videomass-Colours/24x24'},
                    'Ubuntu-Dark-Aubergine':  # Videomass icons for all themes
                    {'x48': f'{icodir}/Sign_Icons/48x48_dark',
                     'x16': f'{icodir}/Videomass-Dark/16x16',
                     'x22': f'{icodir}/Videomass-Dark/24x24'},
                    'Ubuntu-Light-Aubergine':  # Videomass icons for all themes
                    {'x48': f'{icodir}/Sign_Icons/48x48_light',
                     'x16': f'{icodir}/Videomass-Light/16x16',
                     'x22': f'{icodir}/Videomass-Light/24x24'},
                    }

        choose = iconames.get(icontheme)  # set appropriate icontheme
        ext = 'svg' if 'wx.svg' in sys.modules else 'png'
        iconset = (self.prg_icon,
                   f"{choose.get('x48')}/icon_videoconversions.{ext}",
                   f"{choose.get('x22')}/convert.{ext}",
                   f"{choose.get('x22')}/properties.{ext}",
                   f"{choose.get('x16')}/playback.{ext}",
                   f"{choose.get('x48')}/icon_concat.{ext}",
                   f"{choose.get('x16')}/preview.{ext}",
                   f"{choose.get('x16')}/edit-clear.{ext}",
                   f"{choose.get('x16')}/addtoprst.{ext}",
                   f"{choose.get('x16')}/transform-scale.{ext}",
                   f"{choose.get('x16')}/transform-crop.{ext}",
                   f"{choose.get('x16')}/transform-rotate.{ext}",
                   f"{choose.get('x16')}/deinterlace.{ext}",
                   f"{choose.get('x16')}/denoise.{ext}",
                   f"{choose.get('x16')}/volanalyze.{ext}",
                   f"{choose.get('x16')}/configure.{ext}",
                   f"{choose.get('x16')}/player-volume.{ext}",
                   f"{choose.get('x48')}/icon_prst_mng.{ext}",
                   f"{choose.get('x16')}/newprf.{ext}",
                   f"{choose.get('x16')}/delprf.{ext}",
                   f"{choose.get('x16')}/editprf.{ext}",
                   f"{choose.get('x22')}/go-previous.{ext}",
                   f"{choose.get('x22')}/go-next.{ext}",
                   f"{choose.get('x16')}/stabilizer.{ext}",
                   f"{choose.get('x16')}/preview_audio.{ext}",
                   f"{choose.get('x16')}/copyprf.{ext}",
                   f"{choose.get('x48')}/icon_slideshow.{ext}",
                   f"{choose.get('x48')}/icon_videopictures.{ext}",
                   f"{choose.get('x16')}/audiotrack.{ext}",
                   f"{choose.get('x16')}/timer.{ext}",
                   f"{choose.get('x16')}/coloreq.{ext}",
                   f"{choose.get('x22')}/stop.{ext}",
                   f"{choose.get('x22')}/home.{ext}",
                   f"{choose.get('x48')}/youtube.{ext}",
                   f"{choose.get('x16')}/playlist.{ext}",
                   f"{choose.get('x22')}/download.{ext}",
                   f"{choose.get('x22')}/statistics.{ext}",
                   f"{choose.get('x22')}/play.{ext}",
                   f"{choose.get('x16')}/subtitles.{ext}",
                   f"{choose.get('x22')}/proc-queue.{ext}",
                   f"{choose.get('x22')}/add-queue.{ext}",
                   f"{choose.get('x22')}/options.{ext}",
                   )
        values = (os.path.join(norm) for norm in iconset)  # normalize pathns

        return dict(zip(keys, values))



# -*- coding: UTF-8 -*-
"""
Name: generic_downloads.py
Porpose: generic network download operation
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.14.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import shutil
import ssl
import urllib.request
from threading import Thread
import requests
import wx
from pubsub import pub


class FileDownloading(Thread):
    """
    'FileDownloading' is a generic network download operation
    via `urllib.request`. It is used to download small files
    and save them on filesystem.

    """

    def __init__(self, url, filename):
        """
        Attributes defined here:
        self.url: file on network
        self.filename: file to download (dirname + filename.ext)
        self.data: returned output of the self.status
        self.status: tuple object with exit status of the process

        """
        self.url = url
        self.filename = filename
        self.data = None
        self.status = None

        Thread.__init__(self)
        self.start()  # start the thread (va in self.run())
    # ----------------------------------------------------------------#

    def run(self):
        """
        Check for new version release
        """
        # HACK fix soon the ssl certificate
        context = ssl._create_unverified_context()
        headers = {'User-Agent': 'Mozilla/5.0'}
        page = urllib.request.Request(self.url, headers=headers)
        try:
            with urllib.request.urlopen(page, context=context) as \
                    response, open(self.filename, 'wb') as out_file:
                shutil.copyfileobj(response, out_file)

            self.status = self.url, None

        except urllib.error.HTTPError as error:
            self.status = None, error

        except urllib.error.URLError as error:
            self.status = None, error

        self.data = self.status

        wx.CallAfter(pub.sendMessage,
                     "RESULT_EVT",
                     status=''
                     )
# ---------------------------------------------------------------------#


def download_bigfile(url, filename):
    """
    network download operation via `requests`. It is used to
    download big files and save them on filesystem.

    """
    # ------------------------------------------
    # this section is an example to get tarball_url name from github.
    # Use `get_github_releases(url, keyname)` function on IO module
    url = 'https://api.github.com/repos/jeanslack/Videomass/releases/latest'
    filename = '/home/gianluca/Modelli/porzione.tar.gz'
    response = requests.get(url, timeout=10)
    tarball = response.json()["tarball_url"]
    # -------------------------------------------

    with requests.get(tarball, stream=True, timeout=10) as dwnl:
        dwnl.raise_for_status()
        with open(filename, 'wb', encoding='utf-8') as fname:
            for chunk in dwnl.iter_content(chunk_size=8192):
                fname.write(chunk)

    return filename



# -*- coding: UTF-8 -*-

# Porpose: Contains test cases for the check_bin object.
# Rev: Oct.04.2020 *PEP8 compatible*

import sys
import platform
import os.path
import unittest

if sys.version_info[0] != 3:
    sys.exit('\nERROR: You are using an unsupported version of Python. '
             'Python3 is required.\n')

PATH = os.path.realpath(os.path.abspath(__file__))
sys.path.insert(0, os.path.dirname(os.path.dirname(PATH)))

try:
    from videomass.vdms_threads.check_bin import ff_conf
except ImportError as error:
    sys.exit(error)


class CheckFFmpegTestCase(unittest.TestCase):
    """Test case for FFmpeg"""

    def setUp(self):
        """Method called to prepare the test fixture"""
        if platform.system() == 'Windows':
            ffmpeg_url = 'ffmpeg.exe'
        else:
            ffmpeg_url = 'ffmpeg'

        self.out = ff_conf(ffmpeg_url, platform.system())

    def test_invalid_executable(self):
        """
        test error with an invalid or not found ffmpeg executable.

        """
        if 'Not found' in self.out[0]:
            self.assertRaises(AssertionError)
            self.assertEqual(self.out[0], 'Not found')

    def test_available_executable(self):
        """
        test with a valid ffmpeg executable installed on path name
        of the environment variables.
        """
        if 'None' in self.out[0]:
            self.assertEqual(self.out[0], 'None')
            self.assertTrue(self.out)


def main():
    unittest.main()


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-
"""
Name: pictures_exporting.py
Porpose: FFmpeg long processing task on save as pictures
Compatibility: Python3, wxPython4 Phoenix (OS Unix-like only)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.24.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
from threading import Thread
import time
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import logwrite
if not platform.system() == 'Windows':
    import shlex


class PicturesFromVideo(Thread):
    """
    This class represents a separate thread for running simple
    single processes to save video sequences as pictures.

    NOTE capturing output in real-time (Windows, Unix):

    https://stackoverflow.com/questions/1388753/how-to-get-output-
    from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1
    """
    # ------------------------------------------------------

    def __init__(self, *args, **kwargs):
        """
        Called from `long_processing_task.topic_thread`.
        Also see `main_frame.switch_to_processing`.

        """
        get = wx.GetApp()  # get videomass wx.App attribute
        self.appdata = get.appset
        self.stop_work_thread = False  # process terminate
        self.fname = kwargs['filename']
        self.outputdir = kwargs['outputdir']  # output directory
        self.cmd = kwargs['args']  # comand set on single pass
        self.duration = kwargs['duration'][0]  # duration list
        self.count = 0  # count first for loop
        self.logfile = args[0]  # log filename
        self.kwa = kwargs

        Thread.__init__(self)
        self.start()  # self.run()

    def run(self):
        """
        Subprocess initialize thread.
        """
        filedone = []
        cmd = (f'"{self.appdata["ffmpeg_cmd"]}" '
               f'{self.kwa["start-time"]} '
               f'{self.kwa["end-time"]} '
               f'{self.appdata["ffmpeg-default-args"]} '
               f'{self.appdata["ffmpeg_loglev"]} '
               f'{self.kwa["pre-input-1"]} '
               f'-i "{self.kwa["filename"]}" '
               f'{self.cmd}'
               )
        count1 = (f'File 1/1\nSource: "{self.fname}"\n'
                  f'Destination: "{self.outputdir}"')
        com = f'{count1}\n\n[COMMAND]:\n{cmd}'

        wx.CallAfter(pub.sendMessage,
                     "COUNT_EVT",
                     count=count1,
                     duration=self.duration,
                     end='CONTINUE',
                     )
        logwrite(com, '', self.logfile)  # write n/n + command only

        if not self.appdata['ostype'] == 'Windows':
            cmd = shlex.split(cmd)
        try:
            with Popen(cmd,
                       stderr=subprocess.PIPE,
                       stdin=subprocess.PIPE,
                       bufsize=1,
                       universal_newlines=True,
                       encoding=self.appdata['encoding'],
                       ) as proc:
                for line in proc.stderr:
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output=line,
                                 duration=self.duration,
                                 status=0,
                                 )
                    if self.stop_work_thread:
                        proc.stdin.write('q')  # stop ffmpeg
                        out = proc.communicate()[1]
                        proc.wait()
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output='STOP',
                                     duration=self.kwa['duration'],
                                     status=1,
                                     )
                        logwrite('', out, self.logfile)
                        time.sleep(1)
                        wx.CallAfter(pub.sendMessage, "END_EVT",
                                     filetotrash=None)
                        return

                if proc.wait():  # error
                    out = proc.communicate()[1]
                    wx.CallAfter(pub.sendMessage,
                                 "UPDATE_EVT",
                                 output='FAILED',
                                 duration=self.kwa['duration'],
                                 status=proc.wait(),
                                 )
                    logwrite('', (f"[VIDEOMASS]: Error Exit Status: "
                                  f"{proc.wait()} {out}"), self.logfile)
                    time.sleep(1)

                else:  # Done
                    filedone.append(self.fname)
                    wx.CallAfter(pub.sendMessage,
                                 "COUNT_EVT",
                                 count='',
                                 duration='',
                                 end='DONE'
                                 )
        except (OSError, FileNotFoundError) as err:
            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=err,
                         duration=0,
                         end='ERROR',
                         )
            logwrite('', err, self.logfile)

        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT", filetotrash=filedone)
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
"""
Name: ydl_mediainfo.py
Porpose: show media streams information through youtube-dl.extract_info
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.09.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import wx
from pubsub import pub


class YdlMediaInfo(wx.Dialog):
    """
    Display streams information from youtube-dl data.

    """
    def __init__(self, data):
        """
        NOTE constructor:: with 'None' not depend from videomass.
        With 'parent, -1' if close videomass also close mediainfo window
        """
        self.data = data
        get = wx.GetApp()  # get data from bootstrap
        colorscheme = get.appset['colorscheme']
        appicon = get.iconset['videomass']

        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
                           )
        # Add widget controls
        self.url_select = wx.ListCtrl(self,
                                      wx.ID_ANY,
                                      style=wx.LC_REPORT
                                      | wx.SUNKEN_BORDER
                                      | wx.LC_SINGLE_SEL,
                                      )
        self.textctrl = wx.TextCtrl(self,
                                    wx.ID_ANY, "",
                                    style=wx.TE_MULTILINE
                                    | wx.TE_READONLY
                                    | wx.TE_RICH2,
                                    )
        self.textctrl.SetBackgroundColour(colorscheme['BACKGRD'])
        self.textctrl.SetDefaultStyle(wx.TextAttr(colorscheme['TXT3']))
        button_close = wx.Button(self, wx.ID_CLOSE, "")

        # ----------------------Properties----------------------#
        self.SetTitle(_('Statistics viewer'))
        self.SetMinSize((640, 400))
        self.url_select.SetMinSize((640, 200))
        self.url_select.InsertColumn(0, _('TITLE SELECTION'), width=250)
        self.url_select.InsertColumn(1, _('URL'), width=500)
        self.textctrl.SetMinSize((640, 300))

        if get.appset['ostype'] == 'Darwin':
            self.url_select.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL,
                                            wx.NORMAL))
            self.textctrl.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL,
                                          wx.BOLD))
        else:
            self.url_select.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL,
                                            wx.NORMAL))
            self.textctrl.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL,
                                          wx.NORMAL))
        # ----------------------Layout--------------------------#
        sizer_1 = wx.BoxSizer(wx.VERTICAL)
        sizer_1.Add(self.url_select, 0, wx.ALL | wx.EXPAND, 5)
        grid_sizer_1 = wx.GridSizer(1, 1, 0, 0)
        sizer_1.Add(grid_sizer_1, 1, wx.EXPAND, 0)

        grid_sizer_1.Add(self.textctrl, 0, wx.ALL | wx.EXPAND, 5)
        grid_buttons = wx.GridSizer(1, 1, 0, 0)
        grid_buttons.Add(button_close, 1, wx.ALL, 5)
        sizer_1.Add(grid_buttons, flag=wx.ALIGN_RIGHT | wx.RIGHT, border=0)

        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(appicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetSizer(sizer_1)
        sizer_1.Fit(self)
        self.Layout()

        index = 0
        for url in self.data:
            self.url_select.InsertItem(index, str(url['title']))
            self.url_select.SetItem(index, 1, url['url'])
            index += 1

        # ----------------------Binding (EVT)----------------------#
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.url_select)
        self.Bind(wx.EVT_BUTTON, self.on_close, button_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)
        if self.data:
            self.url_select.Focus(0)  # first line the current line selected
            self.url_select.Select(0, on=1)  # default event selection

    # ----------------------Event handler (callback)----------------------#

    def on_select(self, event):
        """
        show data during items selection

        """
        self.textctrl.Clear()  # delete previous append:

        index = self.url_select.GetFocusedItem()
        item = self.url_select.GetItemText(index)
        text = ('An error has occurred.\n'
                'Unable to parse data from provided URL.')

        for info in self.data:
            if info['title'] == item:
                text = (f"Categories:      {info.get('categories', 'N/A')}\n"
                        f"License:         {info.get('license', 'N/A')}\n"
                        f"Upload Date:     {info.get('upload_date', 'N/A')}\n"
                        f"Uploader:        {info.get('uploader', 'N/A')}\n"
                        f"View Count:      {info.get('view', 'N/A')}\n"
                        f"Like Count:      {info.get('like', 'N/A')}\n"
                        f"Dislike Count:   {info.get('dislike', 'N/A')}\n"
                        f"Average Rating:  {info.get('avr_rat', 'N/A')}\n"
                        f"ID:              {info.get('id', 'N/A')}\n"
                        f"Duration:        {info.get('duration', 'N/A')}\n"
                        f"Description:     {info.get('description', 'N/A')}\n"
                        )
        self.textctrl.AppendText(text)
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        Destroy mini frame
        """
        pub.sendMessage("DESTROY_ORPHANED_YTDLP", msg='YdlMediaInfo')



# -*- coding: UTF-8 -*-
"""
Name: ffmpeg_help.py
Porpose: Show a window to search for FFmpeg help topics
Compatibility: Python3, wxPython4
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: March.14.2023
Code checker: flake8, pylint

This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

"""
import re
import wx
from pubsub import pub
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup
from videomass.vdms_io import io_tools


class FFmpegHelp(wx.Dialog):
    """
    Search and view all the FFmpeg help options.
    It has a real-time string search filter and is
    case-sensitive by default, but it is possible to
    ignore upper and lower case by activating the
    corresponding checkbox.

    """
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    HELPTOPIC = (_('Contextual help based on the argument entered in the '
                   'additional text field.\nEach argument consists of the '
                   'type and a type-specific name.\n\n'
                   'For the list of encoders click the "Encoders" button.\n'
                   'For the list of decoders click on the "Decoders" button.\n'
                   'For the muxers and demuxers click on the "Muxers and '
                   'Demuxers" button.\nFor the filters, select "Show '
                   'available filters" in the drop down menu.\n'
                   'For the bsf, select "Show available bitstream '
                   'filters" in the drop down menu.\nFor the protocols, '
                   'select "Show available protocols" in the drop down '
                   'menu.\n\n'
                   'Some examples:'
                   ))
    EXAMPLES = ('\n\n\tencoder = libvpx-vp9'
                '\n\tdecoder = libaom-av1'
                '\n\tmuxer = matroska'
                '\n\tdemuxer = matroska'
                '\n\tfilter = scale'
                '\n\tbsf = av1_frame_merge'
                '\n\tprotocol = bluray'
                )
    ARGS_OPT = {_("Help topic"): ['--help', ''],
                _("List basic options"): ['-h', ''],
                _("List more options"): ['-h', 'long'],
                _("List all options (very long)"): ['-h', 'full'],
                _("Show available devices"): ['-devices'],
                _("Show available bit stream filters"): ['-bsfs'],
                _("Show available protocols"): ['-protocols'],
                _("Show available filters"): ['-filters'],
                _("Show available pixel formats"): ['-pix_fmts'],
                _("Show available audio sample formats"): ['-sample_fmts'],
                _("Show available color names"): ['-colors'],
                _("List sources of the input device"): ['-sources'],
                _("List sinks of the output device"): ['-sinks'],
                _("Show available HW acceleration methods"): ['-hwaccels'],
                _("Show license"): ['-L'],
                }
    MAIN_CHOICES = tuple(ARGS_OPT.keys())
    ARGS = ('encoder', 'decoder', 'muxer', 'demuxer',
            'filter', 'bsf', 'protocol')

    def __init__(self, parent, OS):
        """
        The list of topics in the combo box is part of the
        section given by the -h option on the FFmpeg command line.

        Mode:
            with 'None' not depend from parent:
            wx.Frame.__init__(self, None)

            With parent, -1:
            wx.Frame.__init__(self, parent, -1)
            if close videomass also close parent window

        """
        self.parent = parent
        self.row = None  # output text from `io_tools.findtopic(topic)'
        get = wx.GetApp()  # get data from bootstrap
        colorscheme = get.appset['colorscheme']
        vidicon = get.iconset['videomass']

        wx.Dialog.__init__(self, None,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           | wx.DIALOG_NO_PARENT
                           )
        sizer = wx.BoxSizer(wx.VERTICAL)
        boxshow = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(boxshow, 0, wx.ALL, 5)

        btn_readme = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        btn_readme.SetBackgroundColour(wx.Colour(FFmpegHelp.LGREEN))
        btn_readme.SetForegroundColour(wx.Colour(FFmpegHelp.BLACK))
        boxshow.Add(btn_readme, 0,)

        btn_conf = wx.Button(self, wx.ID_ANY,
                             label=_("Show configuration"),
                             name='configuration',
                             )
        btn_encoders = wx.Button(self, wx.ID_ANY,
                                 label=_("Encoders"),
                                 name='encoders',
                                 )
        btn_decoders = wx.Button(self, wx.ID_ANY,
                                 label=_("Decoders"),
                                 name='decoders',
                                 )
        btn_formats = wx.Button(self, wx.ID_ANY,
                                label=_("Muxers and Demuxers"),
                                name='formats',
                                )
        for button in (btn_conf, btn_encoders, btn_decoders, btn_formats):
            button.Bind(wx.EVT_BUTTON, self.on_show_extra)
            boxshow.Add(button, 0, wx.LEFT, 5)
        self.textlist = wx.TextCtrl(self, wx.ID_ANY,
                                    "",
                                    # size=(550,400),
                                    style=wx.TE_READONLY
                                    | wx.TE_MULTILINE
                                    | wx.TE_RICH2
                                    | wx.HSCROLL,
                                    )
        self.textlist.SetBackgroundColour(colorscheme['BACKGRD'])
        self.textlist.SetDefaultStyle(wx.TextAttr(colorscheme['TXT3']))
        if OS == 'Darwin':
            self.textlist.SetFont(wx.Font(12, wx.MODERN, wx.NORMAL, wx.NORMAL))
        else:
            self.textlist.SetFont(wx.Font(9, wx.MODERN, wx.NORMAL, wx.NORMAL))
        self.textlist.AppendText(FFmpegHelp.HELPTOPIC)
        self.textlist.AppendText(FFmpegHelp.EXAMPLES)
        self.textlist.SetInsertionPoint(0)
        sizer.Add(self.textlist, 1, wx.EXPAND | wx.ALL, 5)
        self.cmbx_main = wx.ComboBox(self, wx.ID_ANY,
                                     choices=FFmpegHelp.MAIN_CHOICES,
                                     style=wx.CB_DROPDOWN
                                     | wx.CB_READONLY,
                                     )
        self.cmbx_main.SetSelection(0)
        self.cmbx_main.SetToolTip(_("help topic list"))

        sizerselect = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(sizerselect, 0)
        sizerselect.Add(self.cmbx_main, 0, wx.ALL, 5)
        self.cmbx_arg = wx.ComboBox(self, wx.ID_ANY,
                                    choices=FFmpegHelp.ARGS,
                                    size=(140, -1),
                                    style=wx.CB_DROPDOWN
                                    | wx.CB_READONLY,
                                    )
        self.cmbx_arg.SetSelection(0)
        sizerselect.Add(self.cmbx_arg, 0, wx.ALL, 5)
        self.text_topic = wx.TextCtrl(self, wx.ID_ANY,
                                      "", size=(140, -1),
                                      style=wx.TE_PROCESS_ENTER
                                      )
        sizerselect.Add(self.text_topic, 0, wx.ALL, 5)
        self.text_topic.Enable()
        self.text_topic.SetToolTip(_("Type the argument here and confirm "
                                     "with the enter key on your keyboard"))
        boxsearch = wx.BoxSizer(wx.HORIZONTAL)
        self.search_str = wx.SearchCtrl(self,
                                        wx.ID_ANY,
                                        size=(400, 30),
                                        style=wx.TE_PROCESS_ENTER,
                                        )
        self.search_str.SetToolTip(_("The search function allows you to find "
                                     "entries in the current topic"))
        boxsearch.Add(self.search_str, 0, wx.ALL, 0)
        self.case = wx.CheckBox(self, wx.ID_ANY, (_("Ignore case")))
        self.case.SetToolTip(_("Ignore case distinctions: characters with "
                               "different case will match."
                               ))
        boxsearch.Add(self.case, 0, wx.LEFT | wx.ALIGN_CENTER_VERTICAL, 5)
        sizer.Add(boxsearch, 0, wx.ALL, 5)
        # bottom layout
        grid = wx.GridSizer(1, 1, 0, 0)
        btn_close = wx.Button(self, wx.ID_CLOSE, "")
        grid.Add(btn_close, 0)
        sizer.Add(grid, 0, wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)
        self.SetTitle(_("FFmpeg help topics"))
        self.SetMinSize((750, 550))
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(vidicon, wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)

        self.SetSizer(sizer)
        self.Fit()
        self.Layout()
        # --------------- EVT
        self.Bind(wx.EVT_BUTTON, self.on_help, btn_readme)
        if not hasattr(wx, 'EVT_SEARCH'):
            self.Bind(wx.EVT_SEARCHCTRL_SEARCH_BTN,
                      self.on_search_strings, self.search_str)
        else:  # is wxPython >= 4.1
            self.Bind(wx.EVT_SEARCH, self.on_search_strings, self.search_str)

        if not hasattr(wx, 'EVT_SEARCH_CANCEL'):
            self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.on_delete,
                      self.search_str)
        else:  # is wxPython >= 4.1
            self.Bind(wx.EVT_SEARCH_CANCEL, self.on_delete, self.search_str)
        self.Bind(wx.EVT_COMBOBOX, self.on_selected, self.cmbx_main)
        self.Bind(wx.EVT_TEXT, self.on_search_strings, self.search_str)
        self.Bind(wx.EVT_TEXT, self.on_type_topic, self.text_topic)
        self.Bind(wx.EVT_TEXT_ENTER, self.on_type_enter_key, self.text_topic)
        self.Bind(wx.EVT_CHECKBOX, self.on_ckbx, self.case)
        self.Bind(wx.EVT_BUTTON, self.on_close, btn_close)
        self.Bind(wx.EVT_CLOSE, self.on_close)  # caption X
    # ---------------------------------------------------------#

    def append_text(self, text):
        """
        Set the value to the text control. Note that on
        MS-Windows I've had some problems with rendering
        text color using the SetValue() method, so it seems
        the only chance is to use the Clear() method first
        and then the AppendText() method here.
        """
        self.textlist.Clear()  # delete previous append:
        self.textlist.AppendText(text)
        self.textlist.SetInsertionPoint(0)  # scroll to top
    # --------------------------------------------------------------#

    def on_help(self, event):
        """
        event on button help
        """
        msg = (_("This is a multifunctional search tool useful for local "
                 "help searches\n"
                 "on multiple FFmpeg topics and options. The search control, "
                 "to\nfind occurences on text obtained from the chosen "
                 "argument,\nis very similar to what grep does on the "
                 "Unix shell, but here\nit's done in real time.\n\n"
                 "If you don't find support for a certain codec or other "
                 "feature,\nit's likely that your version of FFmpeg is "
                 "too old or not configured\nwith those features."))
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   msg,
                                   FFmpegHelp.LGREEN,
                                   FFmpegHelp.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # --------------------------------------------------------------#

    def on_show_extra(self, event):
        """
        It can display formats (muxers and demuxers) codecs
        and decoders windows by calling them from the parent
        (main_frame) instead of calling them from the menu bar
        """
        name = event.GetEventObject().GetName()
        if name == 'formats':
            self.parent.get_ffmpeg_formats(self)
        elif name == 'decoders':
            self.parent.get_ffmpeg_decoders(self)
        elif name == 'encoders':
            self.parent.get_ffmpeg_codecs(self)
        elif name == 'configuration':
            self.parent.get_ffmpeg_conf(self)
    # --------------------------------------------------------------#

    def on_type_topic(self, event):
        """
        Event on typing any text on text box
        """
        if not self.text_topic.GetValue().strip():
            FFmpegHelp.ARGS_OPT[_("Help topic")] = ['--help', '']
            self.append_text(FFmpegHelp.HELPTOPIC + FFmpegHelp.EXAMPLES)
    # --------------------------------------------------------------#

    def on_type_enter_key(self, event):
        """
        Event when user press enter key on self.text_topic box.
        """
        self.search_str.Clear()
        arg = self.cmbx_arg.GetStringSelection()
        argstr = self.text_topic.GetValue().strip()
        if not argstr:
            return
        topic = ['--help', ] + [f'{arg}={argstr}']
        FFmpegHelp.ARGS_OPT[_("Help topic")] = topic
        self.row = io_tools.findtopic(topic)
        if self.row:
            self.append_text(self.row)
        else:
            self.append_text(_("\n  ..Nothing available"))
    # --------------------------------------------------------------#

    def on_selected(self, event):
        """
        Event selecting topic in the `cmbx_main`
        """
        topic = FFmpegHelp.ARGS_OPT[self.cmbx_main.GetValue()]
        self.search_str.Clear()

        if topic[0] == "--help":  # from _(Help topic) only
            self.text_topic.Enable()
            self.cmbx_arg.Enable()
            if not self.text_topic.GetValue().strip():
                self.append_text(FFmpegHelp.HELPTOPIC + FFmpegHelp.EXAMPLES)
            else:
                self.on_type_enter_key(self)
            return

        self.text_topic.Disable()
        self.cmbx_arg.Disable()
        self.row = io_tools.findtopic(topic)
        if self.row:
            self.append_text(self.row)
        else:
            self.append_text(_("\n  ..Nothing available"))
    # --------------------------------------------------------------#

    def on_search_strings(self, event, by_event=False):
        """
        Whenever text is entered it search the string typed
        on the search control and find on the current `self.row`
        output. The result is very similar to what grep does on
        the Unix like shell, i.e:
            `ffmpeg -*some option* | grep somestring`

        or if checkbox is True:

            `ffmpeg -*some option* | grep -i somestring`
        """
        if by_event:  # see `on_ckbx`
            is_string = self.search_str.GetValue()
        else:  # in all other cases
            is_string = event.GetString()

        # ---> only cmbx_choice selection 0
        if (self.cmbx_main.GetSelection()
                == 0 and not self.text_topic.GetValue().split()):
            if not is_string:
                self.on_type_topic(self)
                return
            self.append_text(_("\n  ..Nothing available"))
            return
        # --- end

        if self.row and not is_string:
            self.append_text(self.row)  # load all lines
            return

        if self.row and is_string:  # specified search (like grep does)
            find = []
            if self.case.GetValue():  # case insensitive: Hello/hello

                for lines in self.row.split('\n'):
                    if re.search(is_string, lines, re.IGNORECASE):
                        find.append(f"{lines}\n")
            else:  # is case sensitive
                for lines in self.row.split('\n'):  # case sensitive
                    if is_string in lines:
                        find.append(f"{lines}\n")
            if not find:
                self.append_text(_("\n  ..Not found"))
            else:
                self.append_text(' '.join(find))
        return
    # --------------------------------------------------------------#

    def on_ckbx(self, event):
        """
        This event updates the quick search with
        or without case sensitivity
        """
        self.on_search_strings(self, True)
    # --------------------------------------------------------------#

    def on_delete(self, event):
        """
        This event occurs when user click the small icon along
        the search text field to clear previus text entered.
        It seems does nothing, but it needed.
        During deletion, the "on_search_strings" call
        is generated automatically.
        """
        return
    # --------------------------------------------------------------#

    def on_close(self, event):
        """
        Destroy this window
        """
        pub.sendMessage("DESTROY_ORPHANED_WINDOWS", msg='HelpTopic')



# -*- coding: UTF-8 -*-
"""
File Name: checkup.py
Porpose: input/output file check
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: April.17.2023
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx
from videomass.vdms_dialogs.list_warning import ListWarning


def check_inout(file_sources, file_dest):
    """
    check for overwriting and file/dir existence.
    return tuple([file_sources], [file_dest]).
    return None otherwise.

    """
    # --------------- CHECK FOR OVERWRITING:
    files_ow = []  # already exist file LIST
    files_exist = []
    for files in file_dest:
        if os.path.exists(files):
            files_ow.append(f'"{files}"')

    if files_ow:
        msg = _('Files already exist, do you want to overwrite them?')
        with ListWarning(None,
                         dict.fromkeys(files_ow, _('Already exist')),
                         caption=_('Please confirm'),
                         header=msg,
                         buttons='CONFIRM',
                         ) as log:
            if log.ShowModal() != wx.ID_YES:
                return None

    # --------------- CHECK FOR MSSING FILES:
    for fln in file_sources:
        if not os.path.isfile(os.path.abspath(fln)):
            files_exist.append(f'"{fln}"')
    if files_exist:
        with ListWarning(None,
                         dict.fromkeys(files_exist, _('Not found')),
                         caption=_('Error list'),
                         header=_('File(s) not found'),
                         buttons='OK',
                         ) as log:
            log.ShowModal()
        return None
    # --------------- CHECK FOR EXISTING DIRECTORIES:
    for drn in file_dest:
        drn = os.path.abspath(os.path.dirname(drn))
        if not os.path.isdir(drn):
            wx.MessageBox(_('Output folder does not exist:\n\n"{}"\n').format(
                          drn), _('Videomass - Error!'), wx.ICON_ERROR
                          )
            return None

    return (file_sources, file_dest)
# ------------------------------------------------------------------------#


def check_files(file_sources,
                dir_destin,
                same_destin,
                suffix,
                extout,
                outputnames
                ):
    """
    Build the full path filename for output files.
    return ([file_sources], [file_dest])
    None otherwise.

    """
    if not file_sources:
        return None

    file_dest = []  # output path names (not files)

    for path, fname in zip(file_sources, outputnames):
        dirname = os.path.dirname(path)

        if not extout:  # uses more extensions (copy formats)
            basename = os.path.basename(path)
            copyext = os.path.splitext(basename)[1]  # ext
            if same_destin:
                pathname = os.path.join(dirname, f'{fname}{suffix}{copyext}')
                file_dest.append(pathname)
            else:
                pathname = os.path.join(dir_destin, f'{fname}{copyext}')
                file_dest.append(pathname)

        else:  # uses one extension for all output
            if same_destin:
                pathname = os.path.join(dirname, f'{fname}{suffix}.{extout}')
                file_dest.append(pathname)
            else:
                pathname = os.path.join(dir_destin, f'{fname}.{extout}')
                file_dest.append(pathname)

    return check_inout(file_sources, file_dest)



# -*- coding: UTF-8 -*-
"""
Name: epilogue.py
Porpose: shows dialog box before start process
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: July.17.2022
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

"""
import os
import wx
import wx.lib.scrolledpanel as scrolled


class Formula(wx.Dialog):
    """
    Show a dialog box before run process.
    """
    def __init__(self, parent, *args, **kwargs):

        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        colorscheme = self.appdata['colorscheme']
        self.movetotrash = args[1]
        self.emptylist = args[2]

        wx.Dialog.__init__(self, parent, -1,
                           style=wx.DEFAULT_DIALOG_STYLE
                           | wx.RESIZE_BORDER
                           )
        sizbase = wx.BoxSizer(wx.VERTICAL)
        panelscroll = scrolled.ScrolledPanel(self, wx.ID_ANY,
                                             size=args[0],
                                             style=wx.TAB_TRAVERSAL
                                             | wx.BORDER_THEME,
                                             name="panelscr",
                                             )
        sizbase.Add(panelscroll, 1, wx.ALL | wx.EXPAND, 5)
        label1 = wx.StaticText(panelscroll, wx.ID_ANY, kwargs['key'])
        label2 = wx.StaticText(panelscroll, wx.ID_ANY, kwargs['val'])
        panelscroll.SetBackgroundColour(colorscheme['BACKGRD'])
        label1.SetForegroundColour(colorscheme['TXT3'])
        label2.SetForegroundColour(colorscheme['TXT1'])
        grid_pan = wx.BoxSizer(wx.HORIZONTAL)
        grid_pan.Add(label1, 0, wx.ALL
                     | wx.ALIGN_CENTRE_VERTICAL
                     | wx.ALIGN_CENTRE_HORIZONTAL, 5)
        grid_pan.Add(label2, 0, wx.ALL
                     | wx.ALIGN_CENTRE_VERTICAL
                     | wx.ALIGN_CENTRE_HORIZONTAL, 5)

        panelscroll.SetSizer(grid_pan)
        panelscroll.SetAutoLayout(1)
        panelscroll.SetupScrolling()

        lab = (_('When finished, once the operations '
                 'have been completed successfully:'))
        lbl = wx.StaticText(self, label=lab)
        sizbase.Add(lbl, 0, wx.LEFT | wx.TOP, 5)
        sizeropt = wx.BoxSizer(wx.HORIZONTAL)
        sizbase.Add(sizeropt, 0)
        descr = _("Trash the source files")
        self.ckbx_trash = wx.CheckBox(self, wx.ID_ANY, (descr))
        self.ckbx_trash.SetValue(self.movetotrash)
        sizeropt.Add(self.ckbx_trash, 0, wx.ALL, 5)
        descr = _("Clear the File List")
        self.ckbx_del = wx.CheckBox(self, wx.ID_ANY, (descr))
        self.ckbx_del.SetValue(self.emptylist)
        if self.movetotrash:
            self.ckbx_del.Disable()
        sizeropt.Add(self.ckbx_del, 0, wx.ALL, 5)
        btncancel = wx.Button(self, wx.ID_CANCEL, "")
        btnok = wx.Button(self, wx.ID_OK, _("Run"))
        btngrid = wx.FlexGridSizer(1, 2, 0, 0)
        btngrid.Add(btncancel, 0)
        btngrid.Add(btnok, 0, wx.LEFT, 5)
        sizbase.Add(btngrid, flag=wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=5)

        self.SetTitle(_('Videomass - Batch Mode'))
        self.SetMinSize(args[0])
        self.SetSizer(sizbase)
        sizbase.Fit(self)
        self.Layout()
        # ----------------------Binders (EVT)--------------------#
        self.Bind(wx.EVT_CHECKBOX, self.on_file_to_trash, self.ckbx_trash)
        self.Bind(wx.EVT_CHECKBOX, self.on_empty_list, self.ckbx_del)
        self.Bind(wx.EVT_BUTTON, self.on_cancel, btncancel)
        self.Bind(wx.EVT_BUTTON, self.on_ok, btnok)

        # --------------  Event handler (callback)  --------------#

    def on_empty_list(self, event):
        """
        enable/disable empty imported file list.
        """
        if self.ckbx_del.IsChecked():
            self.emptylist = True
        else:
            self.emptylist = False
    # --------------------------------------------------------------------#

    def on_file_to_trash(self, event):
        """
        enable/disable "Move file to trash" after successful encoding
        """
        trashdir = self.appdata['trashdir_loc']
        if self.ckbx_trash.IsChecked():
            self.movetotrash = True
            self.ckbx_del.SetValue(True)
            self.ckbx_del.Disable()
            self.emptylist = True
            if not os.path.exists(trashdir):
                os.mkdir(trashdir, mode=0o777)
        else:
            self.movetotrash = False
            self.ckbx_del.Enable()
            self.ckbx_del.SetValue(False)
            self.emptylist = False
    # --------------------------------------------------------------------#

    def on_cancel(self, event):
        """
        exit from formula dialog
        """
        # self.Destroy()
        event.Skip()

    def on_ok(self, event):
        """
        get confirmation to proceed
        """
        # self.Destroy()
        event.Skip()

    def getvalue(self):
        """
        This method return values via the getvalue() interface
        from the caller. See the caller for more info and usage.
        """
        return self.movetotrash, self.emptylist



# -*- coding: UTF-8 -*-
"""
Name: main_ytdlp.py
Porpose: window main frame for yt_dlp library
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.09.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
from pubsub import pub
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_ytdlp.ydl_mediainfo import YdlMediaInfo
from videomass.vdms_ytdlp.textdrop import Url_DnD_Panel
from videomass.vdms_ytdlp.youtubedl_ui import Downloader
from videomass.vdms_ytdlp.long_task_ytdlp import LogOut
from videomass.vdms_io import io_tools
from videomass.vdms_sys.settings_manager import ConfigManager
from videomass.vdms_ytdlp.ydl_preferences import Ytdlp_Options
if wx.GetApp().appset['use-downloader']:
    import yt_dlp


class MainYtdl(wx.Frame):
    """
    This is the main frame top window for panels implementation.
    """
    # set status bar colours in html rappresentation:
    ORANGE = '#f28924'
    WHITE = '#fbf4f4'
    # -------------------------------------------------------------#

    def __init__(self, appdata, parent=None):
        """
        If not data_url ytDownloader panel will be disabled
        """
        get = wx.GetApp()
        self.appdata = appdata
        self.icons = get.iconset
        self.data_url = []  # list of urls in text box
        self.changed = True  # previous list is different from new one
        self.infomediadlg = False  # media info dialog

        wx.Frame.__init__(self, parent, -1, style=wx.DEFAULT_FRAME_STYLE)

        # ---------- panel instances:
        self.parent = parent
        self.ytDownloader = Downloader(self)
        self.textDnDTarget = Url_DnD_Panel(self)
        self.ProcessPanel = LogOut(self)
        # hide panels
        self.ProcessPanel.Hide()
        self.ytDownloader.Hide()
        self.textDnDTarget.Show()
        # Layout toolbar buttons:
        mainSizer = wx.BoxSizer(wx.VERTICAL)  # sizer base global
        # Layout external panels:
        mainSizer.Add(self.textDnDTarget, 1, wx.EXPAND)
        mainSizer.Add(self.ytDownloader, 1, wx.EXPAND)
        mainSizer.Add(self.ProcessPanel, 1, wx.EXPAND)

        # ----------------------Set Properties----------------------#
        self.SetTitle("Videomass - YouTube Downloader")
        icon = wx.Icon()
        icon.CopyFromBitmap(wx.Bitmap(self.icons['videomass'],
                                      wx.BITMAP_TYPE_ANY))
        self.SetIcon(icon)
        self.SetMinSize((850, 560))
        self.SetSizer(mainSizer)
        self.Fit()
        self.SetSize(tuple(self.appdata['main_ytdl_size']))
        self.Move(tuple(self.appdata['main_ytdl_pos']))
        # menu bar
        self.videomass_menu_bar()
        # tool bar main
        self.videomass_tool_bar()
        # status bar
        self.sb = self.CreateStatusBar(1)
        self.statusbar_msg(_('Ready'), None)
        # disable some toolbar item
        [self.toolbar.EnableTool(x, False) for x in (20, 22, 23, 24)]
        self.Layout()
        # ---------------------- Binding (EVT) ----------------------#
        self.Bind(wx.EVT_BUTTON, self.on_outputdir)
        self.Bind(wx.EVT_CLOSE, self.on_closefromcaption)

        pub.subscribe(self.check_modeless_window, "DESTROY_ORPHANED_YTDLP")
        pub.subscribe(self.process_terminated, "PROCESS_TERMINATED_YTDLP")

    # -------------------Status bar settings--------------------#

    def statusbar_msg(self, msg, bcolor, fcolor=None):
        """
        Set the status-bar message and color.
        Note that These methods don't always work on every platform.
        Usage:
            - self.statusbar_msg(_('...Finished'))  # no color
            - self.statusbar_msg(_('...Finished'),
                                 bcolor=color,
                                 fcolor=color)  # with colors

        bcolor: background color, fcolor: foreground color
        """
        if self.appdata['ostype'] == 'Linux':
            if not bcolor:
                self.sb.SetBackgroundColour(wx.NullColour)
                self.sb.SetForegroundColour(wx.NullColour)
            else:
                self.sb.SetBackgroundColour(bcolor)
                self.sb.SetForegroundColour(fcolor)

        self.sb.SetStatusText(msg)
        self.sb.Refresh()
    # ------------------------------------------------------------------#

    def choosetopicRetrieve(self):
        """
        Retrieve to choose topic panel
        """

        if self.ytDownloader.IsShown():
            self.ytDownloader.Hide()
            self.textDnDTarget.Show()

            self.ydlpan.Enable(False)

            self.SetTitle(_('Videomass'))
            self.statusbar_msg(_('Ready'), None)
            self.Layout()
    # ------------------------------------------------------------------#

    def check_modeless_window(self, msg=None):
        """
        Receives a message from a modeless window close event.
        This method is called using pub/sub protocol subscribing
        "DESTROY_ORPHANED_YTDLP".
        """
        if msg == 'YdlMediaInfo':
            self.infomediadlg.Destroy()
            self.infomediadlg = False
    # ------------------------------------------------------------------#

    def destroy_orphaned_window(self):
        """
        Destroys all orphaned modeless windows,
        ie. on application exit or on editing URLs text.
        """
        if self.infomediadlg:
            self.infomediadlg.Destroy()
            self.infomediadlg = False

    # ---------------------- Event handler (callback) ------------------#

    def on_statistics(self, event):
        """
        Redirect input files at stream_info for media information
        """
        if self.infomediadlg:
            self.infomediadlg.Raise()
            return

        info = []
        if self.data_url:
            info = self.ytDownloader.on_show_statistics()
            if not info:
                return

        self.infomediadlg = YdlMediaInfo(info)
        self.infomediadlg.Show()
    # ------------------------------------------------------------------#

    def on_closefromcaption(self, event):
        """
        Event from EVT_CLOSE binder, offer the user
        the possibility to choose how to close this window.
        """
        if not self.data_url:
            self.on_exit(None, warn=False)
            return
        dlg = wx.MessageDialog(self, _('Do you want to close the active view '
                                       'keeping the data in memory and any '
                                       'background processes?'),
                               _('Please confirm'), wx.ICON_QUESTION
                               | wx.CANCEL | wx.YES_NO)
        res = dlg.ShowModal()
        if res == wx.ID_YES:
            self.on_close(None)
        elif res == wx.ID_NO:
            self.on_exit(None, warn=False)
        else:
            return
    # ------------------------------------------------------------------#

    def on_close(self, event):
        """
        This event only hide the YouTube Downloader child frame.
        """
        self.Hide()
    # ------------------------------------------------------------------#

    def on_exit(self, event, warn=True):
        """
        This event destroy the YouTube Downloader child frame.
        """
        if self.ProcessPanel.IsShown():
            if self.ProcessPanel.thread_type is not None:
                wx.MessageBox(_("There are still active windows with running "
                                "processes, make sure you finish your work "
                                "before exit."),
                              _('Videomass - Warning!'), wx.ICON_WARNING, self)
                return

        if self.data_url and self.appdata['warnexiting'] and warn:
            if wx.MessageBox(_('Are you sure you want to exit this window?\n'
                               'All data will be lost'),
                             _('Quit YouTube Downloader?'), wx.ICON_QUESTION
                             | wx.CANCEL | wx.YES_NO, self) != wx.YES:
                return

        confmanager = ConfigManager(self.appdata['fileconfpath'])
        sett = confmanager.read_options()
        sett['main_ytdl_size'] = list(self.GetSize())
        sett['main_ytdl_pos'] = list(self.GetPosition())
        sett['subtitles_options'] = self.ytDownloader.opt["SUBS"]
        fcodecolwidth = [self.ytDownloader.panel_cod.fcode.GetColumnWidth(0),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(1),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(2),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(3),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(4),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(5),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(6),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(7),
                         self.ytDownloader.panel_cod.fcode.GetColumnWidth(8),
                         ]
        sett['fcode_column_width'] = fcodecolwidth
        confmanager.write_options(**sett)
        self.destroy_orphaned_window()
        self.Destroy()
    # ------------------------------------------------------------------#

    # -------------   BUILD THE MENU BAR  ----------------###

    def videomass_menu_bar(self):
        """
        Make a menu bar. Per usare la disabilitazione di un menu item devi
        prima settare l'attributo self sull'item interessato - poi lo gestisci
        con self.item.Enable(False) per disabilitare o (True) per abilitare.
        Se vuoi disabilitare l'intero top di items fai per esempio:
        self.menuBar.EnableTop(6, False) per disabilitare la voce Help.
        """
        self.menuBar = wx.MenuBar()

        # ----------------------- file menu
        fileButton = wx.Menu()
        dscrp = _("Open destination folder of downloads\tCtrl+D")
        fold_downloads = fileButton.Append(wx.ID_ANY, dscrp)
        fileButton.AppendSeparator()
        closeItem = fileButton.Append(wx.ID_CLOSE, _("Close view\tCtrl+W"),
                                      _("Close the active view keeping the "
                                        "data in memory and any background "
                                        "processes"))
        exitItem = fileButton.Append(wx.ID_EXIT,
                                     _("Quit YouTube Downloader\tCtrl+Q"),
                                     _("Exit the window by deleting all data "
                                       "in memory"))
        self.menuBar.Append(fileButton, _("File"))

        # ------------------ Edit menu
        editButton = wx.Menu()
        dscrp = (_("Paste\tCtrl+V"),
                 _("Paste the copied URLs to clipboard"))
        self.paste = editButton.Append(wx.ID_PASTE, dscrp[0], dscrp[1])
        dscrp = (_("Remove selected URL\tDEL"),
                 _("Remove the selected URL from the list"))
        self.delete = editButton.Append(wx.ID_REMOVE, dscrp[0], dscrp[1])
        dscrp = (_("Clear list\tShift+DEL"),
                 _("Clear the URL list"))
        self.clearall = editButton.Append(wx.ID_CLEAR, dscrp[0], dscrp[1])
        editButton.AppendSeparator()
        self.setupItem = editButton.Append(wx.ID_PREFERENCES,
                                           _("Options\tCtrl+P"),
                                           _("YouTube Downloader options"))
        self.menuBar.Append(editButton, _("Edit"))

        # ------------------ tools menu
        toolsButton = wx.Menu()
        dscrp = (_("Work notes\tCtrl+N"),
                 _("Read and write useful notes and reminders."))
        notepad = toolsButton.Append(wx.ID_ANY, dscrp[0], dscrp[1])
        self.menuBar.Append(toolsButton, _("Tools"))

        # ------------------ View menu
        viewButton = wx.Menu()
        dscrp = (_("Version of yt-dlp"),
                 _("Shows the version in use"))
        self.ydlused = viewButton.Append(wx.ID_ANY, dscrp[0], dscrp[1])
        dscrp = (_("Latest version of yt-dlp"),
                 _("Check the latest version available on github.com"))
        self.ydllatest = viewButton.Append(wx.ID_ANY, dscrp[0], dscrp[1])
        self.menuBar.Append(viewButton, _("View"))
        # -------
        self.SetMenuBar(self.menuBar)

        # -----------------------Binding menu bar-------------------------#
        # ----FILE----
        self.Bind(wx.EVT_MENU, self.openMydownload, fold_downloads)
        self.Bind(wx.EVT_MENU, self.on_close, closeItem)
        self.Bind(wx.EVT_MENU, self.on_exit, exitItem)
        # ----EDIT----
        self.Bind(wx.EVT_MENU, self.textDnDTarget.on_paste, self.paste)
        self.Bind(wx.EVT_MENU, self.textDnDTarget.on_del_url_selected,
                  self.delete)
        self.Bind(wx.EVT_MENU, self.textDnDTarget.delete_all, self.clearall)
        self.Bind(wx.EVT_MENU, self.on_options, self.setupItem)
        # ----TOOLS----
        self.Bind(wx.EVT_MENU, self.reminder, notepad)
        # ---- VIEW ----
        self.Bind(wx.EVT_MENU, self.ydl_used, self.ydlused)
        self.Bind(wx.EVT_MENU, self.ydl_latest, self.ydllatest)

    # --------Menu Bar Event handler (callbacks)

    def openMydownload(self, event):
        """
        Open the download dir with file manager

        """
        io_tools.openpath(self.appdata['ydlp-outputdir'])
    # -------------------------------------------------------------------#

    def reminder(self, event):
        """
        Call `io_tools.openpath` to open a 'user_memos.txt' file
        with default GUI text editor.
        """
        fname = os.path.join(self.appdata['confdir'], 'user_memos.txt')

        if os.path.exists(fname) and os.path.isfile(fname):
            io_tools.openpath(fname)
        else:
            with open(fname, "w", encoding='utf-8') as text:
                text.write("")
            io_tools.openpath(fname)
    # ------------------------------------------------------------------#

    def ydl_used(self, event, msgbox=True):
        """
        check version of youtube-dl used from
        'Version in Use' bar menu
        """
        this = yt_dlp.version.__version__
        if msgbox:
            wx.MessageBox(_("You are using \"yt-dlp\" "
                            "version {0}").format(this),
                          'Videomass', wx.ICON_INFORMATION, self)
            return this
        return None
    # -----------------------------------------------------------------#

    def ydl_latest(self, event):
        """
        check for new version from github.com
        """
        url = "https://api.github.com/repos/yt-dlp/yt-dlp/releases/latest"
        latest = io_tools.get_github_releases(url, "tag_name")
        if latest[0] in ['request error:', 'response error:']:
            wx.MessageBox(f"{latest[0]} {latest[1]}",
                          f"{latest[0]}", wx.ICON_ERROR, self)
            return
        wx.MessageBox(_("\"yt-dlp\": Latest version "
                        "available: {0}").format(latest[0]),
                      "Videomass", wx.ICON_INFORMATION, self)
    # -----------------------------------------------------------------#

    def on_outputdir(self, event):
        """
        Button event to set a new destination directory.
        """
        dialdir = wx.DirDialog(self, _("Choose Destination"),
                               self.appdata['ydlp-outputdir'],
                               wx.DD_DEFAULT_STYLE
                               )
        if dialdir.ShowModal() == wx.ID_OK:
            getpath = self.appdata['getpath'](dialdir.GetPath())
            self.textDnDTarget.on_file_save(getpath)
            self.appdata['ydlp-outputdir'] = getpath

            confmanager = ConfigManager(self.appdata['fileconfpath'])
            sett = confmanager.read_options()
            sett['ydlp-outputdir'] = self.appdata['ydlp-outputdir']
            confmanager.write_options(**sett)
            dialdir.Destroy()
    # ------------------------------------------------------------------#

    def on_options(self, event):
        """
        Setting dialog event.
        """
        with Ytdlp_Options(self) as set_up:
            if set_up.ShowModal() == wx.ID_OK:
                self.textDnDTarget.on_file_save(self.appdata['ydlp-outputdir'])
    # ------------------------------------------------------------------#

    # -----------------  BUILD THE TOOL BAR  --------------------###

    def toolbar_pos(self):
        """
        Return the toolbar style
        """
        if self.appdata['toolbarpos'] == 0:  # on top
            style = wx.TB_TEXT

        elif self.appdata['toolbarpos'] == 1:  # on bottom
            style = wx.TB_TEXT | wx.TB_BOTTOM

        elif self.appdata['toolbarpos'] == 2:  # on right
            style = wx.TB_TEXT | wx.TB_RIGHT

        elif self.appdata['toolbarpos'] == 3:
            style = wx.TB_TEXT | wx.TB_LEFT

        return style
    # ------------------------------------------------------------------#

    def videomass_tool_bar(self):
        """
        Makes and attaches the toolsBtn bar.
        To enable or disable styles, use method `SetWindowStyleFlag`
        e.g.
            self.toolbar.SetWindowStyleFlag(wx.TB_NODIVIDER | wx.TB_FLAT)
        """
        style = self.toolbar_pos()
        self.toolbar = self.CreateToolBar(style=style)
        bmp_size = (int(self.appdata['toolbarsize']),
                    int(self.appdata['toolbarsize']))
        self.toolbar.SetToolBitmapSize(bmp_size)

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpback = get_bmp(self.icons['previous'], bmp_size)
            bmpnext = get_bmp(self.icons['next'], bmp_size)
            bmpstat = get_bmp(self.icons['statistics'], bmp_size)
            bmpydl = get_bmp(self.icons['download'], bmp_size)
            bmpstop = get_bmp(self.icons['stop'], bmp_size)
            bmpopt = get_bmp(self.icons['options'], bmp_size)

        else:
            bmpback = wx.Bitmap(self.icons['previous'], wx.BITMAP_TYPE_ANY)
            bmpnext = wx.Bitmap(self.icons['next'], wx.BITMAP_TYPE_ANY)
            bmpstat = wx.Bitmap(self.icons['statistics'], wx.BITMAP_TYPE_ANY)
            bmpydl = wx.Bitmap(self.icons['download'], wx.BITMAP_TYPE_ANY)
            bmpstop = wx.Bitmap(self.icons['stop'], wx.BITMAP_TYPE_ANY)
            bmpopt = wx.Bitmap(self.icons['options'], wx.BITMAP_TYPE_ANY)

        self.toolbar.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL,
                                     wx.NORMAL, 0, ""))

        tip = _("Go to the previous panel")
        back = self.toolbar.AddTool(20, _('Back'),
                                    bmpback,
                                    tip, wx.ITEM_NORMAL,
                                    )
        tip = _("Go to the next panel")
        forward = self.toolbar.AddTool(21, _('Next'),
                                       bmpnext,
                                       tip, wx.ITEM_NORMAL,
                                       )
        tip = _("Shows statistics and information")
        self.btn_ydlstatistics = self.toolbar.AddTool(22, _('Statistics'),
                                                      bmpstat,
                                                      tip, wx.ITEM_NORMAL,
                                                      )
        tip = _("YouTube Downloader options")
        options = self.toolbar.AddTool(26, _('Options'),
                                       bmpopt, tip, wx.ITEM_NORMAL)
        tip = _("Start downloading")
        self.run_download = self.toolbar.AddTool(23, _('Download'),
                                                 bmpydl, tip, wx.ITEM_NORMAL,
                                                 )
        tip = _("Stops current process")
        stop = self.toolbar.AddTool(24, _('Stop'), bmpstop,
                                    tip, wx.ITEM_NORMAL,
                                    )
        self.toolbar.Realize()

        # ----------------- Tool Bar Binding (evt)-----------------------#
        self.Bind(wx.EVT_TOOL, self.click_start, self.run_download)
        self.Bind(wx.EVT_TOOL, self.on_back, back)
        self.Bind(wx.EVT_TOOL, self.on_forward, forward)
        self.Bind(wx.EVT_TOOL, self.on_statistics, self.btn_ydlstatistics)
        self.Bind(wx.EVT_TOOL, self.click_stop, stop)
        self.Bind(wx.EVT_TOOL, self.on_options, options)

    # --------------- Tool Bar Callback (event handler) -----------------#

    def on_back(self, event):
        """
        Return to the previous panel.
        """
        if self.ProcessPanel.IsShown():
            self.panelShown()
            return

        if self.ytDownloader.IsShown():
            self.switch_text_import(self)
            return
    # ------------------------------------------------------------------#

    def on_forward(self, event):
        """
        redirect to corresponding next panel
        """
        if self.ytDownloader.IsShown():
            self.switch_to_processing('Viewing last log')
            return
        self.switch_youtube_downloader(self)
        self.changed = False
    # ------------------------------------------------------------------#

    def switch_text_import(self, event):
        """
        Show URLs import panel.
        """
        self.ProcessPanel.Hide()
        self.ytDownloader.Hide()
        self.textDnDTarget.Show()
        (self.delete.Enable(True),
         self.paste.Enable(True),
         self.clearall.Enable(True)
         )
        self.toolbar.EnableTool(21, True)
        [self.toolbar.EnableTool(x, False) for x in (20, 22, 23, 24)]
        self.toolbar.Realize()
        self.Layout()
        self.statusbar_msg(_('Ready'), None)
        self.SetTitle(_('Videomass - List of URLs'))
    # ------------------------------------------------------------------#

    def switch_youtube_downloader(self, event):
        """
        Show youtube-dl downloader panel
        """
        self.ytDownloader.clear_data_list(self.changed)
        self.SetTitle(_('Videomass - YouTube Downloader'))
        self.textDnDTarget.Hide()
        self.ytDownloader.Show()
        (self.delete.Enable(False),
         self.paste.Enable(False),
         self.clearall.Enable(False)
         )
        [self.toolbar.EnableTool(x, True) for x in (20, 21, 22, 23)]
        self.toolbar.EnableTool(24, False)
        self.Layout()
    # ------------------------------------------------------------------#

    def switch_to_processing(self, *args):
        """
        Preparing to processing
        """
        if args[0] == 'Viewing last log':
            self.statusbar_msg(_('Viewing last log'), None)
            [self.toolbar.EnableTool(x, False) for x in (21, 24)]
            [self.toolbar.EnableTool(x, True) for x in (20, 22, 23)]

        elif args[0] == 'YouTube Downloader':
            (self.delete.Enable(False),
             self.paste.Enable(False),
             self.clearall.Enable(False),
             self.setupItem.Enable(False)
             )
            [self.toolbar.EnableTool(x, False) for x in (20, 21, 23, 26)]
            [self.toolbar.EnableTool(x, True) for x in (22, 24)]

        self.SetTitle(_('Videomass - Downloader Message Monitoring'))
        self.textDnDTarget.Hide()
        self.ytDownloader.Hide()
        self.ProcessPanel.Show()
        self.Layout()
        self.ProcessPanel.topic_thread(args, self.data_url)
    # ------------------------------------------------------------------#

    def click_start(self, event):
        """
        By clicking on Download buttons, calls the
        `ytDownloader.on_start()` method, which calls the
        'switch_to_processing' method above.
        """
        if self.ytDownloader.IsShown() or self.ProcessPanel.IsShown():
            if not self.data_url:
                self.switch_text_import(self)
                return
            self.ytDownloader.on_start()
            return
    # ------------------------------------------------------------------#

    def click_stop(self, event):
        """
        Stop/Abort the current process
        """
        if self.ProcessPanel.IsShown():
            if self.ProcessPanel.thread_type:
                self.ProcessPanel.on_stop()
    # ------------------------------------------------------------------#

    def process_terminated(self, msg):
        """
        Process report terminated. This method is called using
        pub/sub protocol. see `long_processing_task.end_proc()`)
        """
        self.toolbar.EnableTool(20, True)
        self.toolbar.EnableTool(24, False)
        self.toolbar.EnableTool(26, True)
        self.toolbar.EnableTool(23, True)
        self.setupItem.Enable(True)

        if self.appdata['shutdown']:
            self.parent.auto_shutdown()
        elif self.appdata['auto_exit']:
            self.parent.auto_exit()
    # ------------------------------------------------------------------#

    def panelShown(self):
        """
        Clicking 'Back button' from the `long_processing_task` panel,
        retrieval at previous panel (see `switch_to_processing`
        method above).
        """
        self.ProcessPanel.Hide()
        self.switch_youtube_downloader(self)
        self.Layout()



# -*- coding: UTF-8 -*-
"""
Name: concatenate.py
Porpose: A simple concat demuxer UI
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyleft - 2024 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.25.2024
Code checker: flake8, pylint

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx
import wx.lib.agw.hyperlink as hpl
from videomass.vdms_dialogs.widget_utils import NormalTransientPopup
from videomass.vdms_utils.utils import integer_to_time
from videomass.vdms_io.checkup import check_files
from videomass.vdms_dialogs.epilogue import Formula


def compare_media_param(data):
    """
    This function expects json data from FFprobe to checks
    that the indexed streams of each item in the list have
    the same codec, video size and audio sample rate in order
    to ensure correct file concatenation.
    Returns an error message if any error found,
    Returns None otherwise.
    """
    if len(data) == 1:
        return ('error',
                _('At least two files are required to perform concatenation.'))
    com = {}
    mediatype = []
    for streams in data:
        name = streams.get('format').get('filename')
        com[name] = {}
        for items in streams.get('streams'):
            mediatype.append(items.get('codec_type'))
            if items.get('codec_type') == 'video':
                com[name][items.get('index')] = [items.get('codec_name')]
                size = f"{items.get('width')}x{items.get('height')}"
                com[name][items.get('index')].append(size)
            if items.get('codec_type') == 'audio':
                com[name][items.get('index')] = [items.get('codec_name')]
                com[name][items.get('index')].append(items.get('sample_rate'))
    if not com:
        return ('error', _('Invalid data found'))

    totest = list(com.values())[0]
    if not all(val == totest for val in com.values()):
        return ('error',
                _('The files do not have the same "codec_types", '
                  'same "sample_rate" or same "width" or "height". '
                  'Unable to proceed.'))
    return None, mediatype[0]
# -------------------------------------------------------------------------


class Conc_Demuxer(wx.Panel):
    """
    A simple concat demuxer UI to set media files
    concatenation using concat demuxer,
    see <https://ffmpeg.org/ffmpeg-formats.html#concat>

    """
    LGREEN = '#52ee7d'
    BLACK = '#1f1f1f'
    MSG_1 = _("\n- The concatenation function is performed only with "
              "Audio files or only with Video files."
              "\n\n- The order of concatenation depends on the order in "
              "which the files were added."
              "\n\n- The output file name will have the same name as the "
              "first file added (also depends on the\n"
              "  settings made in the preferences dialog or the renaming "
              "functions used)."
              "\n\n- Video files must have exactly same streams, same "
              "codecs and same\n  width/height, but can be wrapped in "
              "different container formats."
              "\n\n- Audio files must have exactly the same formats, "
              "same codecs with equal sample rate.")

    # ----------------------------------------------------------------#

    def __init__(self, parent):
        """
        This is a panel impemented on MainFrame
        """
        get = wx.GetApp()  # get data from bootstrap
        self.appdata = get.appset
        self.cachedir = self.appdata['cachedir']
        self.parent = parent  # parent is the MainFrame
        self.args = ''
        self.duration = None
        self.ext = None
        self.mediatype = None

        wx.Panel.__init__(self, parent, -1, style=wx.BORDER_THEME)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((20, 20))
        self.btn_help = wx.Button(self, wx.ID_ANY, _("Read me"), size=(-1, -1))
        self.btn_help.SetBackgroundColour(wx.Colour(Conc_Demuxer.LGREEN))
        self.btn_help.SetForegroundColour(wx.Colour(Conc_Demuxer.BLACK))
        sizer.Add(self.btn_help, 0, wx.ALL, 5)
        sizer.Add((20, 20))
        sizer_link2 = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(sizer_link2)
        self.lbl_msg3 = wx.StaticText(self, wx.ID_ANY,
                                      label=_("For more details, see the "
                                              "Videomass User Guide:")
                                      )
        if self.appdata['GETLANG'] in self.appdata['SUPP_LANGs']:
            lang = self.appdata['GETLANG'].split('_')[0]
            page = (f"https://jeanslack.github.io/Videomass/"
                    f"Pages/User-guide-languages/{lang}/1-User_"
                    f"Interface_Overview_{lang}.pdf")
        else:
            page = ("https://jeanslack.github.io/Videomass/"
                    "Pages/User-guide-languages/en/1-User_"
                    "Interface_Overview_en.pdf"
                    )
        link2 = hpl.HyperLinkCtrl(self, -1, ("1.4 Concatenate media files "
                                             "(demuxer)"), URL=page)
        sizer_link2.Add(self.lbl_msg3, 0, wx.ALL | wx.EXPAND, 5)
        sizer_link2.Add(link2)
        sizer_link1 = wx.BoxSizer(wx.HORIZONTAL)
        sizer.Add(sizer_link1)
        self.lbl_msg2 = wx.StaticText(self, wx.ID_ANY,
                                      label=_("For more information, "
                                              "visit the official FFmpeg "
                                              "documentation:")
                                      )
        link1 = hpl.HyperLinkCtrl(self, -1, "3.4 concat",
                                  URL="https://ffmpeg.org/ffmpeg-formats."
                                      "html#concat"
                                  )
        sizer_link1.Add(self.lbl_msg2, 0, wx.ALL | wx.EXPAND, 5)
        sizer_link1.Add(link1)
        self.SetSizer(sizer)

        if self.appdata['ostype'] == 'Darwin':
            self.lbl_msg2.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
            self.lbl_msg3.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            self.lbl_msg2.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))
            self.lbl_msg3.SetFont(wx.Font(8, wx.SWISS, wx.NORMAL, wx.NORMAL))

        self.Bind(wx.EVT_BUTTON, self.on_help, self.btn_help)
    # ---------------------------------------------------------

    def on_help(self, event):
        """
        event on button help
        """
        win = NormalTransientPopup(self,
                                   wx.SIMPLE_BORDER,
                                   Conc_Demuxer.MSG_1,
                                   Conc_Demuxer.LGREEN,
                                   Conc_Demuxer.BLACK)

        # Show the popup right below or above the button
        # depending on available screen space...
        btn = event.GetEventObject()
        pos = btn.ClientToScreen((0, 0))
        sz = btn.GetSize()
        win.Position(pos, (0, sz[1]))

        win.Popup()
    # ---------------------------------------------------------

    def on_start(self):
        """
        Builds FFmpeg command arguments

        """
        fsource = self.parent.file_src
        ftext = os.path.join(self.cachedir, 'tmp', 'flist.txt')

        diff = compare_media_param(self.parent.data_files)
        if diff[0] == 'error':
            wx.MessageBox(diff[1], _('Videomass - Error!'),
                          wx.ICON_ERROR, self)
            return

        self.mediatype = diff[1]
        textstr = []
        self.ext = os.path.splitext(self.parent.file_src[0])[1].split('.')[1]
        self.duration = sum(self.parent.duration)
        for f in self.parent.file_src:
            escaped = f.replace(r"'", r"'\''")  # need escaping some chars
            textstr.append(f"file '{escaped}'")
        self.args = (f'"{ftext}" -map 0:v? -map_chapters 0 '
                     f'-map 0:s? -map 0:a? -map_metadata 0 -c copy')

        with open(ftext, 'w', encoding='utf-8') as txt:
            txt.write('\n'.join(textstr))

        checking = check_files((fsource[0],),
                               self.appdata['outputdir'],
                               self.appdata['outputdir_asinput'],
                               self.appdata['filesuffix'],
                               self.ext,
                               self.parent.outputnames
                               )
        if not checking:  # User changing idea or not such files exist
            return
        newfile = checking[1]

        self.build_args(self.parent.file_src, newfile[0])
    # -----------------------------------------------------------

    def build_args(self, filesrc, newfile):
        """
        Redirect to processing

        """
        logname = 'Concatenate Media File.log'

        kwargs = {'logname': logname, 'type': 'concat_demuxer',
                  'source': filesrc, 'destination': newfile, 'args': self.args,
                  'nmax': len(filesrc), 'duration': self.duration,
                  'start-time': '', 'end-time': '',
                  'preset name': 'Concatenate media files',
                  }
        keyval = self.update_dict(newfile, os.path.dirname(newfile))
        ending = Formula(self, (700, 170),
                         self.parent.movetotrash,
                         self.parent.emptylist,
                         **keyval,
                         )
        if ending.ShowModal() == wx.ID_OK:
            (self.parent.movetotrash,
             self.parent.emptylist) = ending.getvalue()
            self.parent.switch_to_processing(kwargs["type"],
                                             logname,
                                             datalist=kwargs)
    # -----------------------------------------------------------

    def update_dict(self, newfile, destdir):
        """
        Update information before send to epilogue

        """
        lenfile = len(self.parent.file_src)
        dur = integer_to_time(self.duration)
        dest = os.path.join(destdir, newfile)

        keys = (_("Items to concatenate\nFile destination\nOutput Format"
                  "\nOutput multimedia type\nDuration"
                  ))
        vals = (f"{lenfile}\n{dest}\n{self.ext}\n"
                f"{self.mediatype}\n{dur}")

        return {'key': keys, 'val': vals}
